const jq = require('node-jq');
const { env, drgWithAttach, environment, environmentFromService } = require('./util');

// BTR Data Access calls BTR Services in DRG and returns specified results

async function HasTreatmentWill(attach, PatientCPR) {
    id = 'btr.HasTreatmentWill'
    const body = {
        id: id,
        role: 'test.citizen',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    let hasTreatmentWillResponse = await drgWithAttach(attach, body);

    // Returning the boolean telling if there is a treatment will or not
    return hasTreatmentWillResponse.body.response.main.Envelope.Body
        .HasTreatmentWillResponse.willExists._text;
}

async function GetTreatmentWill(attach, PatientCPR) {
    id = 'btr.GetTreatmentWill'
    const body = {
        id: id,
        role: 'test.citizen',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    let treatmentWillResponse = await drgWithAttach(attach, body);

    // Returning treatment will component
    return treatmentWillResponse.body.response.main.Envelope.Body
        .GetTreatmentWillResponse.ClinicalDocument.component.structuredBody
        .component.section.entry.observation.value;
}

async function DeleteTreatmentWill(attach, PatientCPR) {
    id = 'btr.DeleteTreatmentWill'
    const body = {
        id: id,
        role: 'test.citizen',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    await drgWithAttach(attach, body);
}

async function CreateTreatmentWill(attach, PatientCPR) {
    id = 'btr.CreateTreatmentWill'
    const body = {
        id: id,
        role: 'test.citizen',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
            noLifeProlongingIfDying: 'false',
            noLifeProlongingIfSeverelyDegraded: 'false',
            noLifeProlongingIfSeverePain: 'true',
            noForcedTreatmentIfIncapable: 'true',
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    const response = await drgWithAttach(attach, body);

    return response;
}

async function CreateTreatmentWillAdmin(attach, PatientCPR) {
    id = 'btrAdmin.CreateTreatmentWill'
    const body = {
        id: id,
        role: 'test.annebonde',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
            noLifeProlongingIfDying: 'true',
            noLifeProlongingIfSeverelyDegraded: 'true',
            noLifeProlongingIfSeverePain: 'true',
            noForcedTreatmentIfIncapable: 'true',
        },
        'auth-attributes': {},
    };
    await drgWithAttach(attach, body);
}

async function DeleteTreatmentWillAdmin(attach, PatientCPR) {
    id = 'btrAdmin.DeleteTreatmentWill'
    const body = {
        id: id,
        role: 'test.annebonde',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {},
    };
    await drgWithAttach(attach, body);
}

async function UpdateTreatmentWillAdmin(attach, PatientCPR) {
    id = 'btrAdmin.UpdateTreatmentWill'
    const body = {
        id: id,
        role: 'test.annebonde',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
            noLifeProlongingIfDying: 'false',
            noLifeProlongingIfSeverelyDegraded: 'true',
            noLifeProlongingIfSeverePain: 'false',
            noForcedTreatmentIfIncapable: 'false',
        },
        'auth-attributes': {},
    };
   const response = await drgWithAttach(attach, body);

    return response;
}

async function GetTreatmentWillAdmin(attach, PatientCPR) {
    id = 'btrAdmin.GetTreatmentWill'
    const body = {
        id: id,
        role: 'test.annebonde',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,

        },
        'auth-attributes': {},
    };
    await drgWithAttach(attach, body);
}

module.exports = {
    HasTreatmentWill,
    GetTreatmentWill,
    DeleteTreatmentWill,
    CreateTreatmentWill,
    CreateTreatmentWillAdmin,
    DeleteTreatmentWillAdmin,
    UpdateTreatmentWillAdmin,
    GetTreatmentWillAdmin,
};
