const superagent = require('superagent');

const drgs = {
    // to run locally, create a file in /cucumber called '.drg-credentials'
    // put in some valid drg credentials with theses two lines:
    // DRG_USER=something
    // DRG_PASSWORD=secret

    // the '.drg-credentials'-file is ignored by Git - we don't want to share the secret passwords!
    // p.s. it is also used further below, for DTG, as they share credentials
    nsp: {
        endpoint: 'https://testkald.nspop.dk/drg/api/execute',
        username: process.env.DRG_USER,
        password: process.env.DRG_PASSWORD,
    },
    intern: {
        endpoint: 'http://docker.intern.arosii.com:8081/drg/api/execute',
        username: 'test',
        password: 'test',
    },
};
const _drg = drgs[process.env['DRG']] || drgs.nsp;
const drgAgent = superagent
    .agent()
    .auth(_drg.username, _drg.password)
    .accept('application/json');
const drg = function () {
    return drgAgent.post(_drg.endpoint);
};
async function drgWithAttach(attach, body) {
    let response;
    try {
        response = await drg().send(body);
        // Sometimes the DRG will give a 200, even though the response fails, so we have an
        // additional check, if the response has a faultstring
        if (response.body.code != 200 || response.body.response.main.Envelope.Body.Fault) {
            attach('Faultstring: \n' +
                JSON.stringify(
                    response.body.response.main.Envelope.Body.Fault.faultstring,
                    null,
                    2
                ),
                'application/json'
            );
        }
    } catch (err) {
        attach(JSON.stringify(err, null, 2), 'application/json');
        throw err;
    }
    return response;
}
let jenkinsEnv = process.env.ENV
let testEnvironment = null
const patient_cpr = process.env.PATIENT_CPR;
const number_of_relatives = process.env.NUMBER_OF_RELATIVES;

const _dtg = {
    endpoint: 'https://stamdata.nspop.dk/dtg-webservice/',
    username: process.env.DRG_USER,
    password: process.env.DRG_PASSWORD,
};
const dtgAgent = superagent
    .agent()
    .auth(_dtg.username, _dtg.password)
    .accept('application/json');
const dtg = function (url) {
    return dtgAgent.post(_dtg.endpoint + url);
};
async function dtgWithAttach(attach, url, body) {
    let response;
    try {
        response = await dtg(url).send(body);
    } catch (err) {
        attach(JSON.stringify(err, null, 2), 'application/json');
        throw err;
    }
    return response;
}

const services = require('../config/services.json');
const serviceFromReadable = new Map();
const environmentFromService = new Map();

services.forEach(element => {
    serviceFromReadable.set(element.readable, element.drg)
    environmentFromService.set(element.drg, element.environment)
})

const wsdlAgent = superagent.agent();
async function getWsdlWithAttach(url, attach) {
    try {
        const response = await wsdlAgent.get(url);
        return response;
    } catch (error) {
        attach(JSON.stringify(error, null, 2), 'application/json');
        throw error;
    }
}
async function getWsdl(url) {
    try {
        const response = await wsdlAgent.get(url);
        return response;
    } catch (error) {
        throw error;
    }
}

//Tilføjet nedenstående for SORES implementeringen
const _soresEndpoint = {
    'test1': 'http://test1.ekstern-test.nspop.dk:8080/sores/v2',
    'test2': 'http://test2.ekstern-test.nspop.dk:8080/sores/v2',
};
const soresAgent = superagent
    .agent()
    .accept('application/json');

const sores = function (url) {
    const endpoint = _soresEndpoint[simpleEnvironment()];
    return soresAgent.post(endpoint + url);
};

async function soresWithAttach(attach, url, body) {
    let response;
    try {
        response = await sores(url).send(body);
    } catch (err) {
        attach(JSON.stringify(err, null, 2), 'application/json');
        throw err;
    }
    return response;
}

function simpleEnvironment() {
    const findEnvironment = (string) => {
        if (string && string.includes('test1')) return 'test1';
        if (string && string.includes('test2')) return 'test2';
        return null;
    }
    let result = findEnvironment(testEnvironment);
    if (result) {
        return result;
    }
    result = findEnvironment(jenkinsEnv);
    if (result) {
        return result;
    }
    return 'test1';
}

function environment(defaultEnv) {
    //Hvis env er sat i test casen
    if (testEnvironment != null) {
        return testEnvironment
    //Hvis env er sat af jenkins
    } else if (jenkinsEnv != null) {
        if (jenkinsEnv.endsWith('NSP')){
            return jenkinsEnv
        } else {
            return jenkinsEnv + ' ' + defaultEnv
        }
    //Ellers så bruger vi default env fra services.json
    } else {
        return 'test1 ' + defaultEnv
    }
}

//Funktion så vi kan ændre env midt-test
function setEnvironment(env) {
    testEnvironment = env
}

//Funktion så vi kan reset env når en ny test starter
function resetEnvironment() {
    testEnvironment = null
}

module.exports = {
    drg,
    dtg,
    jenkinsEnv,
    patient_cpr,
    number_of_relatives,
    drgWithAttach,
    dtgWithAttach,
    getWsdl,
    getWsdlWithAttach,
    sores,
    soresWithAttach,
    environment,
    setEnvironment,
    resetEnvironment,
    serviceFromReadable,
    environmentFromService
};
