const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const { expect, delay } = require('../util');
const { GetSorEntity, GetSorEntityByProvider, IsSorEntityDescendantOfSorEntity, MapShakToSor, InstitutionOwnerCvrNumberByShak, InstitutionOwnerCvrNumber, EntityNameLookup, SearchAllChildren } = require('dataaccess/sores');

const jq = require('node-jq');

//henter dato angivet i testcasen og gemmer den
Given('dato er sat til {string}', function (date) {
    this.date = date;
});

//////////////////           SORES EntityNameLookup         ////////////////// 

Given('jeg har følgende søgekriterier:', function (dataTable) {
    const criteria = dataTable.hashes()[0];
    this.searchCriteria = {
        matchEntityName: criteria.matchName,
        cvr: criteria.cvr,
        entityTypeNames: [criteria.entityType]
    };
});

When('jeg kalder EntityNameLookup operationen', async function () {

    this.response = await EntityNameLookup(this.attach, this.matchEntityName, this.cvr, this.entityTypeNames);
    console.log(this.response.body);
});

Then('returneres følgende enheder for {string}:', function (searchTerm, dataTable) {
    const resultData = this.response.body[searchTerm];
    expect(resultData).to.exist;

    const expectedData = dataTable.hashes().map(row => ({
        sorId: parseInt(row.sorId),
        from: row.from,
        name: row.name,
        healthInstitutionSorId: parseInt(row.healthInstitutionSorId),
        parentSorId: parseInt(row.parentSorId)
    }));

    // Verificer at antallet af resultater matcher
    expect(resultData.length).to.equal(expectedData.length);

    // For hver forventet række
    expectedData.forEach(expected => {
        const actual = resultData.find(p => p.sorId === expected.sorId);
        expect(actual).to.exist;

        expect(actual.sorId, `sorId for ${expected.name} matcher ikke`)
            .to.equal(expected.sorId);
        expect(actual.from, `from for ${expected.name} matcher ikke`)
            .to.equal(expected.from);
        expect(actual.name, `name for ${expected.name} matcher ikke`)
            .to.equal(expected.name);
        expect(actual.healthInstitutionSorId, `healthInstitutionSorId for ${expected.name} matcher ikke`)
            .to.equal(expected.healthInstitutionSorId);
        expect(actual.parentSorId, `parentSorId for ${expected.name} matcher ikke`)
            .to.equal(expected.parentSorId);
    });
});

Then('der {string} flere resultater', function (condition) {
    const expectedValue = condition === 'findes';
    expect(this.response.body.AdditionalRecordsExist,
        `AdditionalRecordsExist burde være ${expectedValue}`)
        .to.equal(expectedValue);
});








//////////////////           SORES InstitutionOwnerCvrNumberByShak         ////////////////// 

Given('jeg har følgende liste af SOR:', function (dataTable) {
    this.sorIds = dataTable.hashes().map(row => row.sorIds);
});


When('jeg kalder InstitutionOwnerCvrNumber operationen', async function () {

    this.response = await InstitutionOwnerCvrNumber(this.attach, this.sorIds);
    console.log(this.response.body);
});

Given('jeg har følgende liste af SHAK IDs:', function (dataTable) {
    this.shakIds = dataTable.hashes().map(row => row.shakIds);
});


When('jeg kalder InstitutionOwnerCvrNumberByShak operationen', async function () {

    this.response = await InstitutionOwnerCvrNumberByShak(this.attach, this.shakIds);
    console.log(this.response.body);
});

// Fælles Then fixture
Then('returneres følgende CVR numre:', function (dataTable) {
    const rows = dataTable.hashes();

    // Bestem hvilken type ID det er baseret på kolonnerne i tabellen
    const isShak = 'shakId' in rows[0];
    const idField = isShak ? 'shakId' : 'sorId';

    rows.forEach(row => {
        const id = row[idField];
        const expectedCvrNumber = row.cvrNumber;
        const actualCvrNumber = this.response.body[id];

        expect(actualCvrNumber,
            `CVR nummer for ${isShak ? 'SHAK' : 'SOR'} ID ${id} matcher ikke`)
            .to.equal(expectedCvrNumber);
    });
});

//////////////////           SORES MapShakToSor         ////////////////// 

Given('jeg har følgende SHAK ID:', function (dataTable) {
    this.shakId = dataTable.hashes()[0].shakId;
});

When('jeg kalder MapShakToSor operationen', async function () {


    this.response = await MapShakToSor(this.attach, this.shakId);
    console.log(this.response.body);
});

Then('returneres for SHAK {string} følgende resultat:', function (shakId, dataTable) {
    const resultData = this.response.body[shakId];
    expect(resultData).to.exist;

    // Konverter datatabellen til et array af objekter
    const expectedData = dataTable.hashes().map(row => ({
        sorId: parseInt(row.sorId),
        from: row.from,
        healthInstitutionSorId: parseInt(row.healthInstitutionSorId)
    }));

    // Verificer at antallet af resultater matcher
    expect(resultData.length, `Forventet ${expectedData.length} resultater, men fik ${resultData.length}`)
        .to.equal(expectedData.length);

    // For hver forventet række
    expectedData.forEach(expected => {
        const actual = resultData.find(p => p.sorId === expected.sorId);
        expect(actual, `Kunne ikke finde række med sorId ${expected.sorId}`).to.exist;

        // Verificer hver enkelt værdi
        expect(actual.sorId, `sorId matcher ikke for rækken`)
            .to.equal(expected.sorId);
        expect(actual.from, `from dato matcher ikke for rækken`)
            .to.equal(expected.from);
        expect(actual.healthInstitutionSorId, `healthInstitutionSorId matcher ikke for rækken`)
            .to.equal(expected.healthInstitutionSorId);
    });
});





//////////////////           SORES IsSorEntityDescendantOfSorEntity         ////////////////// 



Given('jeg har følgende SOR ID par:', function (dataTable) {
    this.sorIdPairs = dataTable.hashes().map(row => [row.childId, row.ancestorId]);
});

When('jeg kalder IsSorEntityDescendantOfSorEntity operationen', async function () {

    this.response = await IsSorEntityDescendantOfSorEntity(this.attach, this.sorIdPairs);
    console.log(this.response.body);
});

Then('returneres følgende resultat:', function (dataTable) {
    const expectedResults = dataTable.hashes();

    expectedResults.forEach(expected => {
        const key = `child=${expected.childId}, possibleAncestor=${expected.ancestorId}`;
        const actualValue = this.response.body[key];
        const expectedValue = expected.isDescendant.toLowerCase() === 'true';

        expect(actualValue,
            `Forventet ${expectedValue} for ${key}, men fik ${actualValue}`)
            .to.equal(expectedValue);
    });
});

//////////////////           SORES SearchAllChildren           ////////////////// 

When('jeg kalder SearchAllChildren operationen', async function () {
    const res = await SearchAllChildren(this.attach, this.sorIds);
    this.responses.push(res);
});

Then('returneres for SOR {string} følgende resultat:', async function (sorId, dataTable) {
    const expectedResults = dataTable.hashes();
    let response = this.getLastResponse().body[sorId]

    for (var k = 0; k < Object.entries(response); k++) {
        let data = {
            'sorId': response.sorId,
            'from': k.from,
            'name': k.name,
            'institutionOwnerSorId': k.institutionOwnerSorId,
            'shakId': k.shakId,
            'healthInstitutionSorId': k.healthInstitutionSorId,
            'parentSorId': k.parentSorId
        }
        expect(response[k].sorId).to.equal(expectedResults[k].sorId);
        expect(response[k].from).to.equal(expectedResults[k].from);
        expect(response[k].name).to.equal(expectedResults[k].name);
        expect(response[k].institutionOwnerSorId).to.equal(expectedResults[k].institutionOwnerSorId);
        expect(response[k].shakId).to.equal(expectedResults[k].shakId);
        expect(response[k].healthInstitutionSorId).to.equal(expectedResults[k].healthInstitutionSorId);
        expect(response[k].parentSorId).to.equal(expectedResults[k].parentSorId);
    }
});



//////////////////           SORES GetSorEntityByProvider           ////////////////// 


//kalder GetSorEntityByProvider med den/de ydernumre der er angivet, samt evt dato
When('jeg kalder GetSorEntityByProvider operationen', async function () {

    this.response = await GetSorEntityByProvider(this.attach, this.providerIds);
    console.log(this.response.body);
});



//får listen af ydernumre fra testcasen
Given('jeg har følgende liste af Ydernumre:', function (dataTable) {
    this.providerIds = dataTable.hashes().map(row => row.providerIds);
});


Then('returneres for {string} følgende resultat:', function (providerId, dataTable) {
    // Hent response data for det specifikke provider ID
    const providerData = this.response.body[providerId];
    expect(providerData).to.exist;

    // Konverter datatabellen til et array af objekter
    const expectedData = dataTable.hashes().map(row => ({
        sorId: parseInt(row.sorId),
        from: row.from,
        name: row.name.replace(/"/g, ''),
        institutionOwnerSorId: parseInt(row.institutionOwnerSorId),
        healthInstitutionSorId: parseInt(row.healthInstitutionSorId),
        providerId: row.providerId.replace(/"/g, ''),
        parentSorId: parseInt(row.parentSorId)
    }));

    // Verificer at antallet af resultater matcher
    expect(providerData).to.have.lengthOf(expectedData.length);

    // For hver forventet række
    expectedData.forEach(expected => {
        // Find den tilsvarende række i response data
        const actual = providerData.find(p => p.sorId === expected.sorId);

        // Verificer at rækken findes
        expect(actual).to.exist;

        // Verificer hver enkelt værdi
        expect(actual.sorId, `sorId for ${expected.name} matcher ikke`)
            .to.equal(expected.sorId);
        expect(actual.from, `from for ${expected.name} matcher ikke`)
            .to.equal(expected.from);
        expect(actual.name, `name for ${expected.name} matcher ikke`)
            .to.equal(expected.name);
        expect(actual.institutionOwnerSorId, `institutionOwnerSorId for ${expected.name} matcher ikke`)
            .to.equal(expected.institutionOwnerSorId);
        expect(actual.healthInstitutionSorId, `healthInstitutionSorId for ${expected.name} matcher ikke`)
            .to.equal(expected.healthInstitutionSorId);
        expect(actual.providerId, `providerId for ${expected.name} matcher ikke`)
            .to.equal(expected.providerId);
        expect(actual.parentSorId, `parentSorId for ${expected.name} matcher ikke`)
            .to.equal(expected.parentSorId);
    });
});









//////////////////           SORES GETSOREntity           ////////////////// 

//får listen af sorkoder fra testcasen
Given('jeg har følgende liste af SOR IDs:', function (dataTable) {
    this.sorIds = dataTable.hashes().map(row => row.sorId);
});



//kalder GetSorEntity med de sorværdier der er angivet, samt evt dato
When('jeg kalder GetSorEntity operationen', async function () {

    this.response = await GetSorEntity(this.attach, this.sorIds, this.date);
    console.log(this.response);
});

Then('skal responset indeholde information for alle {int} SOR enheder', function (antal) {
    expect(Object.keys(this.response.body)).to.have.lengthOf(antal);
});

//sørger for at sammenholde det sordata der hentes med det der er angivet i testcasen. 
Then('{string} skal have følgende data:', function (entityName, dataTable) {
    const expectedFields = dataTable.hashes();
    const entity = Object.values(this.response.body)
        .find(e => e.name === entityName);

    expect(entity, `Entity med navn ${entityName} blev ikke fundet`).to.exist;

    expectedFields.forEach(field => {
        const actualValue = entity[field.felt]?.toString();
        expect(actualValue, `Felt '${field.felt}' matcher ikke`)
            .to.equal(field.værdi);
    });
});

//sammenligner fejlbesked angivet i testcase med den der modtages
Then('SORES retursvar fejltekst er {string}', function (expectedErrorMessage) {
    // Trim whitespace og sammenlign
    const actualMessage = this.response.text.trim();
    const expectedMsg = expectedErrorMessage.trim();

    // Log værdier for debugging
    console.log('Actual (length):', actualMessage.length, 'Value:', actualMessage);
    console.log('Expected (length):', expectedMsg.length, 'Value:', expectedMsg);

    // Sammenlign med mere detaljer hvis det fejler
    try {
        expect(actualMessage).to.equal(expectedMsg);
    } catch (error) {
        // Log karakterkoder for at identificere forskelle
        console.log('Actual chars:', [...actualMessage].map(c => c.charCodeAt(0)));
        console.log('Expected chars:', [...expectedMsg].map(c => c.charCodeAt(0)));
        throw error;
    }
});

Then('skal responset indeholde tomme objekter', function () {
    // Log det faktiske response for debugging
    console.log('Actual response:', JSON.stringify(this.response.body));
    console.log('Expected SOR IDs:', this.sorIds);

    // Verificer at antallet af properties matcher
    expect(Object.keys(this.response.body).length).to.equal(this.sorIds.length);

    // Verificer at hver værdi er et tomt objekt
    Object.values(this.response.body).forEach(value => {
        expect(value).to.deep.equal({});
    });
});

Then('skal responset kun indeholde én instans af SOR enheden', function () {
    expect(Object.keys(this.response.body)).to.have.lengthOf(1);
    expect(this.response.body).to.have.property('8041000016000');
});
