const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const jq = require('node-jq');
const { expect, decodeJWT, delay, parseRelativeDate } = require('../util');
const { getCBS, cbs } = require('../../dataaccess/cbs');
const dcctype = new Map(require('../../config/dccdatatypes.json'));
const dccvalue = new Map(require('../../config/dccdatavalues.json'));

let dayjs = require('dayjs');
dayjs().format();
let utc = require('dayjs/plugin/utc');
dayjs.extend(utc);
let timezone = require('dayjs/plugin/timezone');
dayjs.extend(timezone);

Then('{string} indeholder {int} tokens i alt', function (ResponseName, Amount) {
    const coronaPasses =
        this.variables.get(ResponseName).body.response.main.Envelope.Body
            .GetCoronaPassResponse.CoronaPasses.Pass;

    let amount;
    if (Array.isArray(coronaPasses)) {
        amount = coronaPasses.length;
        expect(amount).to.eql(Amount);
    } else if (coronaPasses == undefined) {
        throw new Error(`Response is not an array`);
    } else {
        amount = 1;
        expect(amount).to.equal(Amount);
    }
});

Then(
    '{string} indeholder {int} {string} token for {string} {string}',
    async function (ResponseName, Amount, TokenType, TokenKey, TokenValue) {
        const coronaPasses =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .GetCoronaPassResponse.CoronaPasses.Pass;

        let passes;
        if (Array.isArray(coronaPasses)) {
            passes = await jq.run(
                `[.[]|select(.Type._text=="${TokenType}" and (.Properties|select(.Value._text=="${TokenValue}" and .Key._text=="${TokenKey}")))]`,
                coronaPasses,
                { input: 'json', output: 'json' }
            );
            expect(passes.length).to.eql(Amount);
        } else {
            passes = await jq.run(
                `.Type._text=="${TokenType}" and (.Properties|select(.Value._text=="${TokenValue}" and .Key._text=="${TokenKey}"))`,
                coronaPasses,
                { input: 'json', output: 'json' }
            );
            if (passes) {
                expect(1).to.eql(Amount);
            } else {
                expect(0).to.eql(Amount);
            }
        }
    }
);

Then(
    '{string} indeholder {int} EU tokens for {string} {string}',
    async function (ResponseName, Amount, TokenKey, TokenValue) {
        const coronaPasses =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .GetCoronaPassResponse.CoronaPasses.Pass;

        if (Array.isArray(coronaPasses)) {
            let passes = await jq.run(
                `[.[]|select(.Type._text=="EU" and (.Properties[]|select(.Value._text=="${TokenValue}" and .Key._text=="${TokenKey}")))]`,
                coronaPasses,
                { input: 'json', output: 'json' }
            );

            expect(passes.length).to.eql(Amount);
        } else {
            let passes = await jq.run(
                `.Type._text=="EU" and (.Properties[]|select(.Value._text=="${TokenValue}" and .Key._text=="${TokenKey}"))`,
                coronaPasses,
                { input: 'json', output: 'json' }
            );
            if (passes) {
                expect(1).to.eql(Amount);
            } else {
                expect(0).to.eql(Amount);
            }
        }
    }
);

Then(
    '{string} indeholder {int} EU token for {string} {string}',
    async function (ResponseName, Amount, TokenKey, TokenValue) {
        const coronaPasses =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .GetCoronaPassResponse.CoronaPasses.Pass;

        let passes = await jq.run(
            `[.[]|select(.Type._text=="EU" and (.Properties[]|select(.Value._text=="${TokenValue}" and .Key._text=="${TokenKey}")))]`,
            coronaPasses,
            { input: 'json', output: 'json' }
        );

        expect(passes.length).to.eql(Amount);
    }
);

Then(
    '{string} dekodet {string} DCC token for {string} gemmes som {string}',
    async function (ResponseName, TokenType, TokenValue, ResultName) {
        const coronaPasses =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .GetCoronaPassResponse.CoronaPasses.Pass;

        let passes = await jq.run(
            `.[]|select(.Type._text=="${TokenType}" and (.Properties|select(.Value._text=="${TokenValue}")))`,
            coronaPasses,
            { input: 'json', output: 'json' }
        );

        const decodetRespsonse = await decodeJWT(passes.JWT._text);

        this.attach(JSON.stringify(decodetRespsonse, null, 2), 'text/plain');

        this.variables.set(ResultName, decodetRespsonse);
    }
);

Then(
    '{string} indeholder {int} {string} i DCC token',
    function (ResponseName, Amount, Element) {
        const response = this.variables.get(ResponseName);
        const element = dcctype.get(Element);

        if (response['-260']['1'][element] == undefined) {
            expect(0).to.eql(Amount);
        } else {
            expect(response['-260']['1'][element].length).to.eql(Amount);
        }
    }
);

Then(
    '{string} vaccinedose {int} indeholder {string} i feltet {string}',
    async function (ResponseName, Amount, Value, Field) {
        const response = this.variables.get(ResponseName);
        const field = dcctype.get(Field);
        let value = dccvalue.get(Value);

        if (value == undefined) {
            value = Value;
        }

        const vaccines = response['-260']['1'].v;

        let vaccine = await jq.run(`.[]|select(.dn==${Amount})`, vaccines, {
            input: 'json',
            output: 'json',
        });

        expect(vaccine[field]).to.eql(value);
    }
);

Then(
    '{string} vaccinedose {int} indeholder {int} i feltet {string}',
    async function (ResponseName, Amount, Value, Field) {
        const response = this.variables.get(ResponseName);
        const field = dcctype.get(Field);
        let value = dccvalue.get(Value);

        if (value == undefined) {
            value = Value;
        }

        const vaccines = response['-260']['1'].v;

        let vaccine = await jq.run(`.[]|select(.dn==${Amount})`, vaccines, {
            input: 'json',
            output: 'json',
        });

        expect(vaccine[field]).to.eql(value);
    }
);

Then(
    '{string} prøvesvar {int} indeholder {string} i feltet {string}',
    function (ResponseName, Number, Value, Field) {
        const response = this.variables.get(ResponseName);
        const field = dcctype.get(Field);
        let value = dccvalue.get(Value);

        if (value == undefined) {
            value = Value;
        }

        expect(response['-260']['1'].t[Number - 1][field]).to.eql(value);
    }
);

Then(
    '{string} {int} dekodet {string} DCC token for {string} og {string} gemmes som {string}',
    async function (
        ResponseName,
        Number,
        TokenType,
        TokenValue1,
        TokenValue2,
        ResultName
    ) {
        const response =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .GetCoronaPassResponse.CoronaPasses.Pass;

        let passes = await jq.run(
            `[.[]|select(.Type._text=="${TokenType}" and (.Properties[]|select(.Value._text=="${TokenValue1}")) and (.Properties[]|select(.Value._text=="${TokenValue2}")))]`,
            response,
            {
                input: 'json',
                output: 'json',
            }
        );

        let decodetRespsonses = await Promise.all(
            passes.map(async function (element) {
                let decoded = await decodeJWT(element.JWT._text);
                return decoded;
            })
        );

        let decodetRespsonse;

        if (TokenValue2 == 'rec') {
            decodetRespsonse = decodetRespsonses[Number - 1];
        } else if (TokenValue2 == 'vac') {
            decodetRespsonse = await jq.run(
                `.[]|select(.["-260"]["1"].v[]|select(.dn==${Number}))`,
                decodetRespsonses,
                {
                    input: 'json',
                    output: 'json',
                }
            );
        } else if (TokenValue2 == 'lab') {
            decodetRespsonse = await jq.run(
                `.[]|select(.["-260"]["1"].t[]|select(.dn==${Number}))`,
                decodetRespsonses,
                {
                    input: 'json',
                    output: 'json',
                }
            );
        }

        this.attach(JSON.stringify(decodetRespsonse, null, 2), 'text/plain');

        this.variables.set(ResultName, decodetRespsonse);
    }
);

Then(
    'såfremt {string} indeholder {string} DCC tokens for {string} så dekodes og gemmes som {string}',
    async function (ResponseName, TokenType, TokenValue, ResultName) {
        
        const response =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .GetCoronaPassResponse.CoronaPasses.Pass;

        let isArray = Array.isArray(response);

        let tokenTypes;
        if (isArray) {
            tokenTypes = await jq.run(`[.[].Type._text]`, response, {
                input: 'json',
                output: 'json',
            });
        } else {
            tokenTypes = await jq.run(`[.Type._text]`, response, {
                input: 'json',
                output: 'json',
            });
        }

        if (tokenTypes.includes(TokenType)) {
            if (
                TokenType == 'DKINFO' ||
                TokenType == 'DKPLUS' ||
                TokenType == 'DK'
            ) {
                if (isArray) {
                    let tokenTypeResponses = await jq.run(
                        `[.[]|select(.Type._text=="${TokenType}")]`,
                        response,
                        {
                            input: 'json',
                            output: 'json',
                        }
                    );

                    let tokenValues = await jq.run(
                        `[.[].Properties.Value._text]`,
                        tokenTypeResponses,
                        {
                            input: 'json',
                            output: 'json',
                        }
                    );

                    if (tokenValues.includes(TokenValue)) {
                        let passes = await jq.run(
                            `.[]|select(.Type._text=="${TokenType}" and (.Properties|select(.Value._text=="${TokenValue}")))`,
                            response,
                            { input: 'json', output: 'json' }
                        );

                        const decodetRespsonse = await decodeJWT(
                            passes.JWT._text
                        );

                        this.attach(
                            JSON.stringify(decodetRespsonse, null, 2),
                            'text/plain'
                        );

                        this.variables.set(ResultName, decodetRespsonse);
                    } else {
                        this.variables.set(ResultName, '');
                    }
                } else if (!isArray) {
                    let tokenTypeResponses = await jq.run(
                        `[.|select(.Type._text=="${TokenType}")]`,
                        response,
                        {
                            input: 'json',
                            output: 'json',
                        }
                    );

                    let tokenValues = await jq.run(
                        `[.[].Properties.Value._text]`,
                        tokenTypeResponses,
                        {
                            input: 'json',
                            output: 'json',
                        }
                    );

                    if (tokenValues.includes(TokenValue)) {
                        const decodetRespsonse = await decodeJWT(
                            response.JWT._text
                        );

                        this.attach(
                            JSON.stringify(decodetRespsonse, null, 2),
                            'text/plain'
                        );

                        this.variables.set(ResultName, decodetRespsonse);
                    } else {
                        this.variables.set(ResultName, '');
                    }
                } else {
                    this.variables.set(ResultName, '');
                }
            } else if (TokenType == 'EU') {
                let passes = await jq.run(
                    `[.[]|select(.Type._text=="EU" and (.Properties[]|select(.Value._text=="${TokenValue}")))]`,
                    response,
                    { input: 'json', output: 'json' }
                );

                let decodetRespsonse = await Promise.all(
                    passes.map(async function (element) {
                        let decoded = await decodeJWT(element.JWT._text);
                        return decoded;
                    })
                );

                this.attach(
                    JSON.stringify(decodetRespsonse, null, 2),
                    'text/plain'
                );

                this.variables.set(ResultName, decodetRespsonse);
            }
        } else {
            this.attach(JSON.stringify('', null, 2), 'text/plain');
            this.variables.set(ResultName, '');
        }
    }
);

When(
    'såfremt {string} er cachemiss ventes {int} ms og cbs kaldes igen',
    { timeout: -1 },
    async function (ResponseName, ms) {
        const response =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .GetCoronaPassResponse;

        if (response.hasOwnProperty('FutureResult')) {
            if (ms < 0) {
                throw new Error('Delay must be bigger than 0');
            }
            await delay(ms);

            const cbsResponse = await getCBS(
                this.attach,
                this.variables.get(ResponseName).request._data
            );

            this.variables.set(ResponseName, cbsResponse);
            this.responses.push(cbsResponse);

            this.attach('T R U E', 'text/plain');
        } else {
            this.attach('FALSE', 'text/plain');
        }
    }
);

Then(
    '{string} indeholder samme  {string} som {string} DCC token',
    async function (ResponseName1, Element, ResponseName2) {
        const response1 = this.variables.get(ResponseName1);
        const response2 = this.variables.get(ResponseName2);
        const element = dcctype.get(Element);

        let dates1 = await jq.run(
            `[.[]["-260"]["1"].t[].${element}]`,
            response1,
            {
                input: 'json',
                output: 'json',
            }
        );

        let dates2 = await jq.run(
            `[.["-260"]["1"].t[].${element}]`,
            response2,
            {
                input: 'json',
                output: 'json',
            }
        );

        dates1.forEach((element) => {
            expect(dates2.includes(element)).to.be.true;
        });
    }
);

Then(
    '{string} DCC token indeholder {int} tests mere end {string} og de ekstra tests har {string} mindre end dem i EU token',
    async function (ResponseName1, Amount, ResponseName2, Element) {
        const response1 = this.variables.get(ResponseName1);
        const response2 = this.variables.get(ResponseName2);
        const element = dcctype.get(Element);

        let dates1 = await jq.run(
            `[.["-260"]["1"].t[].${element}]`,
            response1,
            {
                input: 'json',
                output: 'json',
            }
        );

        let dates2 = await jq.run(
            `[.[]["-260"]["1"].t[].${element}]`,
            response2,
            {
                input: 'json',
                output: 'json',
            }
        );

        expect(dates1.length).to.equal(dates2.length + Amount);

        for (let i = 0; (i = dates1.length - dates2.length); i++) {
            dates1.sort().shift();
        }

        dates2.sort();

        expect(JSON.stringify(dates1)).to.equal(JSON.stringify(dates2));
    }
);

Then(
    '{string} indeholder {int} {string} i DCC token med {string} {string}',
    async function (ResponseName, Amount, Element, Field, Value) {
        const response = this.variables.get(ResponseName);
        const element = dcctype.get(Element);
        const field = dcctype.get(Field);

        if (Array.isArray(response)) {
            let passes = await jq.run(
                `[.[]|select(.["-260"]["1"].${element}[]|select(.${field}=="${Value}"))]`,
                response,
                {
                    input: 'json',
                    output: 'json',
                }
            );

            expect(passes.length).to.equal(Amount);
        } else if (!(Array.isArray(response) && undefined)) {
            let passes = await jq.run(
                `[.|select(.["-260"]["1"].${element}[]|select(.${field}=="${Value}"))]`,
                response,
                {
                    input: 'json',
                    output: 'json',
                }
            );
            expect(passes.length).to.equal(Amount);
        } else {
            throw new Error('Response is undefined');
        }
    }
);

Then(
    '{string} indeholder {int} tokens i alt før {string} og {int} efter',
    function (ResponseName, AmountBefore, CutoffTime, AmountAfter) {
        const coronaPasses =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .GetCoronaPassResponse.CoronaPasses.Pass;

        let timeNow = dayjs(new Date())
            .tz('Europe/Copenhagen')
            .format('HH.mm.ss');

        if (CutoffTime <= timeNow) {
            console.log(
                'After cutoff time: ' + CutoffTime + ' Time now: ' + timeNow,
                'text/plain'
            );
            this.attach(
                'After cutoff time: ' + CutoffTime + ' Time now: ' + timeNow,
                'text/plain'
            );
            let amount;
            if (Array.isArray(coronaPasses)) {
                amount = coronaPasses.length;
                expect(amount).to.eql(AmountAfter);
            } else if (coronaPasses == undefined) {
                throw new Error(`Response is not an array`);
            } else {
                amount = 1;
                expect(amount).to.equal(AmountAfter);
            }
        } else if (CutoffTime > timeNow) {
            this.attach(
                'Before cutoff time: ' + CutoffTime + ' Time now: ' + timeNow,
                'text/plain'
            );
            let amount;
            if (Array.isArray(coronaPasses)) {
                amount = coronaPasses.length;
                expect(amount).to.eql(AmountBefore);
            } else if (coronaPasses == undefined) {
                throw new Error(`Response is not an array`);
            } else {
                amount = 1;
                expect(amount).to.equal(AmountBefore);
            }
        }
    }
);

Then(
    '{string} indeholder {int} {string} token for {string} {string} før {string} og {int} efter',
    async function (
        ResponseName,
        AmountBefore,
        TokenType,
        TokenKey,
        TokenValue,
        CutoffTime,
        AmountAfter
    ) {
        const coronaPasses =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .GetCoronaPassResponse.CoronaPasses.Pass;

        let localTimeNow = dayjs(new Date())
            .tz('Europe/Copenhagen')
            .format('HH.mm.ss');

        if (CutoffTime <= localTimeNow) {
            this.attach(
                'After cutoff time: ' +
                CutoffTime +
                ' Time now: ' +
                localTimeNow,
                'text/plain'
            );
            let passes;
            if (Array.isArray(coronaPasses)) {
                passes = await jq.run(
                    `[.[]|select(.Type._text=="${TokenType}" and (.Properties|select(.Value._text=="${TokenValue}" and .Key._text=="${TokenKey}")))]`,
                    coronaPasses,
                    { input: 'json', output: 'json' }
                );
                expect(passes.length).to.eql(AmountAfter);
            } else {
                passes = await jq.run(
                    `.Type._text=="${TokenType}" and (.Properties|select(.Value._text=="${TokenValue}" and .Key._text=="${TokenKey}"))`,
                    coronaPasses,
                    { input: 'json', output: 'json' }
                );
                if (passes) {
                    expect(1).to.eql(AmountAfter);
                } else {
                    expect(0).to.eql(AmountAfter);
                }
            }
        } else if (CutoffTime > localTimeNow) {
            this.attach(
                'Before cutoff time: ' +
                CutoffTime +
                ' Time now: ' +
                localTimeNow,
                'text/plain'
            );

            let passes;
            if (Array.isArray(coronaPasses)) {
                passes = await jq.run(
                    `[.[]|select(.Type._text=="${TokenType}" and (.Properties|select(.Value._text=="${TokenValue}" and .Key._text=="${TokenKey}")))]`,
                    coronaPasses,
                    { input: 'json', output: 'json' }
                );
                expect(passes.length).to.eql(AmountBefore);
            } else {
                passes = await jq.run(
                    `.Type._text=="${TokenType}" and (.Properties|select(.Value._text=="${TokenValue}" and .Key._text=="${TokenKey}"))`,
                    coronaPasses,
                    { input: 'json', output: 'json' }
                );
                if (passes) {
                    expect(1).to.eql(AmountBefore);
                } else {
                    expect(0).to.eql(AmountBefore);
                }
            }
        }
    }
);

Then(
    '{string} indeholder {int} EU tokens for {string} {string} før {string} og {int} efter',
    async function (
        ResponseName,
        AmountBefore,
        TokenKey,
        TokenValue,
        CutoffTime,
        AmountAfter
    ) {
        const coronaPasses =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .GetCoronaPassResponse.CoronaPasses.Pass;

        let localTimeNow = dayjs(new Date())
            .tz('Europe/Copenhagen')
            .format('HH.mm.ss');

        if (CutoffTime <= localTimeNow) {
            this.attach(
                'After cutoff time: ' +
                CutoffTime +
                ' Time now: ' +
                localTimeNow,
                'text/plain'
            );

            if (Array.isArray(coronaPasses)) {
                let passes = await jq.run(
                    `[.[]|select(.Type._text=="EU" and (.Properties[]|select(.Value._text=="${TokenValue}" and .Key._text=="${TokenKey}")))]`,
                    coronaPasses,
                    { input: 'json', output: 'json' }
                );

                expect(passes.length).to.eql(AmountAfter);
            } else {
                let passes = await jq.run(
                    `.Type._text=="EU" and (.Properties[]|select(.Value._text=="${TokenValue}" and .Key._text=="${TokenKey}"))`,
                    coronaPasses,
                    { input: 'json', output: 'json' }
                );
                if (passes) {
                    expect(1).to.eql(AmountAfter);
                } else {
                    expect(0).to.eql(AmountAfter);
                }
            }
        } else if (CutoffTime > localTimeNow) {
            this.attach(
                'Before cutoff time: ' +
                CutoffTime +
                ' Time now: ' +
                localTimeNow,
                'text/plain'
            );

            if (Array.isArray(coronaPasses)) {
                let passes = await jq.run(
                    `[.[]|select(.Type._text=="EU" and (.Properties[]|select(.Value._text=="${TokenValue}" and .Key._text=="${TokenKey}")))]`,
                    coronaPasses,
                    { input: 'json', output: 'json' }
                );

                expect(passes.length).to.eql(AmountBefore);
            } else {
                let passes = await jq.run(
                    `.Type._text=="EU" and (.Properties[]|select(.Value._text=="${TokenValue}" and .Key._text=="${TokenKey}"))`,
                    coronaPasses,
                    { input: 'json', output: 'json' }
                );
                if (passes) {
                    expect(1).to.eql(AmountBefore);
                } else {
                    expect(0).to.eql(AmountBefore);
                }
            }
        }
    }
);

Then(
    '{string} indeholder {int} {string} i DCC token før {string} og {int} efter',
    function (ResponseName, AmountBefore, Element, CutoffTime, AmountAfter) {
        const response = this.variables.get(ResponseName);
        const element = dcctype.get(Element);

        let localTimeNow = dayjs(new Date())
            .tz('Europe/Copenhagen')
            .format('HH.mm.ss');

        if (CutoffTime <= localTimeNow) {
            this.attach(
                'After cutoff time: ' +
                CutoffTime +
                ' Time now: ' +
                localTimeNow,
                'text/plain'
            );

            if (response['-260']['1'][element] == undefined) {
                expect(0).to.eql(AmountAfter);
            } else {
                expect(response['-260']['1'][element].length).to.eql(
                    AmountAfter
                );
            }
        } else if (CutoffTime > localTimeNow) {
            this.attach(
                'Before cutoff time: ' +
                CutoffTime +
                ' Time now: ' +
                localTimeNow,
                'text/plain'
            );

            if (response['-260']['1'][element] == undefined) {
                expect(0).to.eql(AmountBefore);
            } else {
                expect(response['-260']['1'][element].length).to.eql(
                    AmountBefore
                );
            }
        }
    }
);

Then(
    '{string} immunitet indeholder dato {string} {string} i feltet {string}',
    async function (ResponseName, Date1, Number, Field) {
        const response = this.variables.get(ResponseName);
        const field = dcctype.get(Field);
        const date = this.variables.get(Date1); //Format: 2022-05-24T22:22:39Z
        const dateFormatted = dayjs(date).tz('Europe/Copenhagen');

        if (Number.includes('M')) {
            Number = Number.substring(1, Number.length - 1);

            expect(response['-260']['1']['r'][0][field]).to.eql(
                dayjs(dateFormatted).add(Number, 'month').format('YYYY-MM-DD')
            );
        } else if (Number.includes('D')) {
            Number = Number.substring(1, Number.length - 1);

            expect(response['-260']['1']['r'][0][field]).to.eql(
                dayjs(dateFormatted).add(Number, 'day').format('YYYY-MM-DD')
            );
        } else if (Number.includes('h')) {
            Number = Number.substring(1, Number.length - 1);
            expect(response['-260']['1']['r'][0][field]).to.eql(
                dayjs(dateFormatted).add(Number, 'hour').format('YYYY-MM-DD')
            );
        }
    }
);

Then(
    '{string} prøvesvar {int} indeholder dato {string} {string} i feltet {string}',
    async function (ResponseName, Amount, Date1, Number, Field) {
        const response = this.variables.get(ResponseName);
        const field = dcctype.get(Field);
        const date = this.variables.get(Date1); //Format: 2022-05-24T22:22:39Z
        const dateFormatted = dayjs(date); //'2022-01-11T09:00:00Z'

        if (Number.includes('M')) {
            Number = Number.substring(1, Number.length - 1);

            this.attach(
                'Expect: ' +
                dayjs(dateFormatted).add(Number, 'month').utc().format()
            );

            expect(response['-260']['1']['t'][Amount - 1][field]).to.eql(
                dayjs(dateFormatted).add(Number, 'month').utc().format()
            );
        } else if (Number.includes('D')) {
            Number = Number.substring(1, Number.length - 1);

            this.attach(
                'Expect: ' +
                dayjs(dateFormatted).add(Number, 'day').utc().format()
            );

            expect(response['-260']['1']['t'][Amount - 1][field]).to.eql(
                dayjs(dateFormatted).add(Number, 'day').utc().format()
            );
        } else if (Number.includes('h')) {
            Number = Number.substring(1, Number.length - 1);

            this.attach(
                'Expect: ' + dateFormatted.utc().add(Number, 'hour').format()
            );

            expect(response['-260']['1']['t'][Amount - 1][field]).to.eql(
                dateFormatted.utc().add(Number, 'hour').format()
            );
        }
    }
);

Then(
    '{string} immunitet indeholder dato {string} i feltet {string} før {string} og {string} efter',
    function (ResponseName, Date1, Field, CutoffTime, Date2) {
        const response = this.variables.get(ResponseName);
        const field = dcctype.get(Field);

        let localTimeNow = dayjs(new Date())
            .tz('Europe/Copenhagen')
            .format('HH.mm.ss');

        if (CutoffTime <= localTimeNow) {
            expect(response['-260']['1']['r'][0][field]).to.eql(
                Date1.substring(0, 10)
            );
        } else if (CutoffTime > localTimeNow) {
            expect(response['-260']['1']['r'][0][field]).to.eql(
                Date2.substring(0, 10)
            );
        }
    }
);

Then(
    '{string} prøvesvar indeholder dato {string} i feltet {string} før {string} og {string} efter',
    function (ResponseName, Date1, Field, CutoffTime, Date2) {
        const response = this.variables.get(ResponseName);
        const field = dcctype.get(Field);

        let localTimeNow = dayjs(new Date())
            .tz('Europe/Copenhagen')
            .format('HH.mm.ss');

        if (CutoffTime <= localTimeNow) {
            expect(response['-260']['1']['t'][0][field]).to.eql(
                Date1.substring(0, 10)
            );
        } else if (CutoffTime > localTimeNow) {
            expect(response['-260']['1']['t'][0][field]).to.eql(
                Date2.substring(0, 10)
            );
        }
    }
);

Given(
    '{string} er {string} fra klokken {string} i dag',
    function (Variable, Value, Clock) {
        const date = dayjs(new Date()).tz('Europe/Copenhagen');
        this.attach('Det er lige nu: ' + dayjs(date).format());
        let dav = dayjs(date)
            .set('hour', parseInt(Clock.substring(0, 2)))
            .set('minute', Clock.substring(3, 5))
            .set('second', Clock.substring(6, 8))
            .format();

        //Handling subtract or add
        if (Value.substring(0, 1) == '-') {
            dav = dayjs(dav).subtract(Value.slice(1, -1), 'hour');
        } else if (Value.substring(0, 1) == '+') {
            dav = dayjs(dav).add(Value.slice(1, -1), 'hour');
        }
        this.variables.set(Variable, dayjs(dav).utc().format());
    }
);

Given(
    'der er lavet et CBS kald med cachemiss på {string} og jobid er gemt som {string}',
    async function (PatientCPR, JobId) {
        PatientCPR = this.replaceVariables(PatientCPR);

        const jobId = await cbs(this.attach, PatientCPR);

        this.variables.set(
            JobId,
            jobId.body.response.main.Envelope.Body.GetCoronaPassResponse
                .FutureResult.JobId._text
        );
    }
);
