const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const jq = require('node-jq');
const { expect } = require('../util');
const dkihetypecode = new Map(require('../../config/dkihetypecodes.json'));
const dkiheformatcode = new Map(require('../../config/dkiheformatcodes.json'));
const dkiheidentifier = new Map(require('../../config/dkiheidentifiers.json'));

// This fixture expects a register response to contain a specific document type
Then(
    'registersvar indeholder dokument med dokumenttype {string}',
    async function (documenttype) {
        // Getting the DK IHE Type Code by the dokument type
        const typeValue = dkihetypecode.get(documenttype);

        // This urn is the IHE identifier for typecode. NSP uses this to identify document type
        // https://gazelle.ihe.net/XDStarClient/rimihe-templates/rimihe-TypeCode_XDSDocumentEntry.html
        const typeCode = 'urn:uuid:f0306f51-975f-434e-a61c-c59651d33983';

        // Getting the documents from the last response
        let documentlist =
            this.getLastResponse().body.response.main.Envelope.Body
                .AdhocQueryResponse.RegistryObjectList.ExtrinsicObject;

        // Making result variable
        let res;
        // If the documents is stored in an array
        if (Array.isArray(documentlist)) {
            // Finding the document with the right type code and type value
            res = await jq.run(
                // Selecting document from an array by type code and value
                `.[] | select(.Classification[] | .__classificationScheme == "${typeCode}" and .__nodeRepresentation == "${typeValue}")`,
                documentlist,
                { input: 'json', output: 'json' }
            );
        } else {
            // Finding the document with the right type code and type value
            res = await jq.run(
                // Selecting document by type code and value
                `. | select(.Classification[] | .__classificationScheme == "${typeCode}" and .__nodeRepresentation == "${typeValue}")`,
                documentlist,
                { input: 'json', output: 'json' }
            );
        }

        // Expecting the document to exist and not be equal to nothing
        expect(res, 'Document must exist').to.not.eql('');

        // Pushing the document
        this.documents.push(res);
    }
);

Then(
    'registersvar som indeholder dokument med dokumenttype {string} gemmes som {string} og {string}',
    async function (documenttype, document1, document2) {
        // Getting the DK IHE Type Code by the dokument type
        const typeValue = dkihetypecode.get(documenttype);

        // This urn is the IHE identifier for typecode. NSP uses this to identify document type
        // https://gazelle.ihe.net/XDStarClient/rimihe-templates/rimihe-TypeCode_XDSDocumentEntry.html
        const typeCode = 'urn:uuid:f0306f51-975f-434e-a61c-c59651d33983';

        // Getting the documents from the last response
        let documentlist =
            this.getLastResponse().body.response.main.Envelope.Body
                .AdhocQueryResponse.RegistryObjectList.ExtrinsicObject;

        // Making result variables
        let res1;
        let res2;
        //As there are two Stamkort documents, we need to save both
        if (documenttype == "Stamkort") {
            //Stamkort v2
            res1 = await jq.run(
                `.[0]| select(.Classification[] | .__classificationScheme == "${typeCode}" and .__nodeRepresentation == "${typeValue}")`,
                documentlist,
                { input: 'json', output: 'json' }
            );

            //Stamkort v3
            res2 = await jq.run(
                `.[1]| select(.Classification[] | .__classificationScheme == "${typeCode}" and .__nodeRepresentation == "${typeValue}")`,
                documentlist,
                { input: 'json', output: 'json' }
            );

            // Expecting the documents to exist and not be equal to nothing
            expect(res1, 'Document must exist').to.not.eql('');
            expect(res2, 'Document must exist').to.not.eql('');

            // Saving the documents
            this.variables.set(document1, res1);
            this.variables.set(document2, res2);
        }
        //Scenarios where the document is not Stamkort
        else if (Array.isArray(documentlist)) {
            // Finding the document with the right type code and type value
            res1 = await jq.run(
                // Selecting document from an array by type code and value
                `.[] | select(.Classification[] | .__classificationScheme == "${typeCode}" and .__nodeRepresentation == "${typeValue}")`,
                documentlist,
                { input: 'json', output: 'json' }
            );
            this.variables.set(document1, res1)
            expect(res1, 'Document must exist').to.not.eql('');
        } else {
            // Finding the document with the right type code and type value
            res1 = await jq.run(
                // Selecting document by type code and value
                `. | select(.Classification[] | .__classificationScheme == "${typeCode}" and .__nodeRepresentation == "${typeValue}")`,
                documentlist,
                { input: 'json', output: 'json' }
            );
            this.variables.set(document1, res1)
            expect(res1, 'Document must exist').to.not.eql('');
        }
    }
);

// This fixture expects a register response to contain a document with a specfic format type
Then(
    'registersvar indeholder dokument med formattype {string}',
    async function (formattype) {
        // Getting the format type code by the format type
        const typeValue = dkiheformatcode.get(formattype);
        // This urn is the IHE identifier for typecode. NSP uses this to identify document type
        // https://gazelle.ihe.net/XDStarClient/rimihe-templates/rimihe-FormatCode_XDSDocumentEntry.html
        const typeCode = 'urn:uuid:a09d5840-386c-46f2-b5ad-9c3699a4309d';

        // Getting the documents from the last response
        let documentlist =
            this.getLastResponse().body.response.main.Envelope.Body
                .AdhocQueryResponse.RegistryObjectList.ExtrinsicObject;

        // Making result variable
        let res;
        // If the documents is stored in an array
        if (Array.isArray(documentlist)) {
            // Finding the document with the right type code and type value
            res = await jq.run(
                // Selecting document from an array by type code and value
                `.[] | select(.Classification[] | .__classificationScheme == "${typeCode}" and .__nodeRepresentation == "${typeValue}")`,
                documentlist,
                { input: 'json', output: 'json' }
            );
        } else {
            // Finding the document with the right type code and type value
            res = await jq.run(
                // Selecting document by type code and value
                `. | select(.Classification[] | .__classificationScheme == "${typeCode}" and .__nodeRepresentation == "${typeValue}")`,
                documentlist,
                { input: 'json', output: 'json' }
            );
        }

        // Expecting the document to exist and not be equal to nothing
        expect(res, 'Document must exist').to.not.eql('');
    }
);

// This fixture expects a document type to have the right service start time
Then(
    'dokumenttype har service start tidspunkt {string}',
    async function (serviceStartTime) {
        // Getting the last document
        let document = this.getLastDocument();

        // Finding place in document where service start time is
        let res = await jq.run(
            // Selecting from the slots array in the document
            `.Slot[] | select(.ValueList.Value._text? == "${serviceStartTime}")`,
            document,
            { input: 'json', output: 'json' }
        );

        // Expects the result not to be nothing
        expect(res, 'Service start time must exist').to.not.eql('');
    }
);

// This fixture expects the document to have the right service stop time
Then(
    'dokumenttype har service stop tidspunkt {string}',
    async function (serviceStopTime) {
        // Getting the last document
        let document = this.getLastDocument();

        // Finding place in document where service stop time is
        let res = await jq.run(
            // Selecting from the slots array in the document
            `.Slot[] | select(.ValueList.Value._text? == "${serviceStopTime}")`,
            document,
            { input: 'json', output: 'json' }
        );

        // Expects the result not to be nothing
        expect(res, 'Service stop time must exist').to.not.eql('');
    }
);

// This fixture expects the document is made on the creation time
Then(
    'dokumenttype har creation tidspunkt {string}',
    async function (creationTime) {
        // Getting the last document
        let document = this.getLastDocument();

        // Finding place in document where creation time is
        let res = await jq.run(
            // Selecting from the slots array in the document
            `.Slot[] | select(.ValueList.Value._text? == "${creationTime}")`,
            document,
            { input: 'json', output: 'json' }
        );

        // Expects the result not to be nothing
        expect(res, 'Creation time must exist').to.not.eql('');
    }
);

// This fixture expects the document to have the right author person
Then('dokumenttype har author person {string}', async function (authorPerson) {
    // Splitting the name from the fixture
    let name = authorPerson.split(' ');

    // First element in array is the first name
    let firstname = name[0];

    // Second element in the array is the last name
    let lastname = name[1];

    // Getting the last document
    let document = this.getLastDocument();

    if (firstname == 'Morten') {
        console.log(JSON.stringify(document, null, 2));
    }

    let res;
    expect(res, 'Author person must exist').to.not.eql('');
});

// This fixture expects a document to have the id
Then(
    '{string} dokumentet har id {string}',
    function (documenttype, documentid) {
        // Expects document id from the last document to equal document id from fixture
        expect(this.getLastDocument().__id).to.eql(documentid);
    }
);

// This fixture expects a document to have the id given the formattype
Then(
    '{string} med formattype {string} dokumentet har id {string}',
    async function (documenttype, formattype, documentid) {
        let id = null;
        const type = formattype;
        let documentlist =
            this.getLastResponse().body.response.main.Envelope.Body
                .AdhocQueryResponse.RegistryObjectList.ExtrinsicObject;

        let res;
        if (Array.isArray(documentlist)) {
            res = await jq.run(
                `.[].Classification[]| select(.__nodeRepresentation == "${type}")`,
                documentlist,
                { input: 'json', output: 'json' }
            );
        } else {
            res = await jq.run(
                `.Classification[]| select(.__nodeRepresentation == "${type}")`,
                documentlist,
                { input: 'json', output: 'json' }
            );
        }
        id = res.__classifiedObject;

        expect(id).to.eql(documentid);

    }
);

// This fixture expects a document to have the id
Then(
    '{string} dokumentet har formatkode {string}',
    function (documenttype, documentformatkode) {
        // Expects document id from the last document to equal document id from fixture
        console.log(JSON.stringify(this.getLastDocument(), 2, null));
        expect(this.getLastDocument().__nodeRepresentation).to.eql(documentformatkode);
    }
);

// This fixture is based on predefined data and this patient CPR contains the data that makes the testcase pass
Given('{string} har alle datatyper i stamkort', function (cpr) {
    cpr = this.replaceVariables(cpr);
    // Expects CPR to be specific CPR
    expect(cpr).to.be.oneOf(['1603900967', '1009639639']);
   
});

// This fixture expects the CDA document to contain an amount of components
Then('CDA indeholder {int} komponenter i alt', function (countElements) {
    // Getting components from the last response
    if (this.getLastResponse().body.response.attachments[0] != null) {
        componentlist = this.getLastResponse().body.response.attachments[0].ClinicalDocument.component
            .structuredBody.component.section.entry;
    } else {
        componentlist = this.getLastResponse().body.response.main.Envelope.Body.RetrieveDocumentSetResponse.
            DocumentResponse.Document._data.ClinicalDocument.component.structuredBody.component.section.entry;
    }

    // Expecting the component list to contain the given number of components
    expect(componentlist.length).to.eql(countElements);
});

// This fixture expects the CDA document to contain an amount of a specific component type
Then(
    'CDA indeholder {int} komponenter af typen {string}',
    async function (countelements, componenttype) {
        let componententries;
        // Finding the documents with an specific component type in the last response
        if (this.getLastResponse().body.response.attachments[0] != null) {
            componententries = await jq.run(
                `[.[]|select(.observation.code|select(."__code"? =="${componenttype}"))]`,
                this.getLastResponse().body.response.attachments[0].ClinicalDocument.component
                    .structuredBody.component.section.entry,
                { input: 'json', output: 'json' }
            );
        } else { //Sometimes the response is not in the attachments
            componententries = await jq.run(
                `[.[]|select(.observation.code|select(."__code"? =="${componenttype}"))]`,
                this.getLastResponse().body.response.main.Envelope.Body.RetrieveDocumentSetResponse.
                    DocumentResponse.Document._data.ClinicalDocument.component.structuredBody.component.section.entry,
                { input: 'json', output: 'json' }
            );
        }
        // Expecting the component list to contain the given number of components
        expect(componententries.length).to.eql(countelements);
    }
);

// This fixture expects the CDA document observation to contain value and verifies that the individual observations fx LivingWill has the specified value
Then(
    'CDA observation {string} indeholder {string}',
    async function (componenttype, value) {
        let componententries;
        // Finding the documents with an specific component type in the last response
        if (this.getLastResponse().body.response.attachments[0] != null) {
            componententries = await jq.run(
                `[.[]|select(.observation.code|select(."__code"? =="${componenttype}"))]`,
                this.getLastResponse().body.response.attachments[0].ClinicalDocument.component
                    .structuredBody.component.section.entry,
                { input: 'json', output: 'json' }
            );
        } else { //Sometimes the response is not in the attachments
            componententries = await jq.run(
                `[.[]|select(.observation.code|select(."__code"? =="${componenttype}"))]`,
                this.getLastResponse().body.response.main.Envelope.Body.RetrieveDocumentSetResponse.
                    DocumentResponse.Document._data.ClinicalDocument.component.structuredBody.component.section.entry,
                { input: 'json', output: 'json' }
            );
        }
        // Expecting the first component's observation value to equal the value from the fixture
        expect(componententries[0].observation.value.__extension).to.eql(value);
    }
);

Then('CDA indeholder tandlæge med navn {string} og telefonnummer {string}', async function (name, phonenumber) {
    let dentist = await jq.run(
        `.[] | select(.td[]?._text == "Tandlæge indtastet")`,
        this.getLastResponse().body.response.main.Envelope.Body.RetrieveDocumentSetResponse.
            DocumentResponse.Document._data.ClinicalDocument.component.structuredBody.component.section.text.table.tbody.tr,
        { input: 'json', output: 'json' }
    );

    let dentistName = await jq.run(
        `.[] | .list[]? | .[]? | select(._text == "${name}")`,
        dentist.td, { input: 'json', output: 'json' }
    );
    let dentistPhone = await jq.run(
        `.[] | .list[]? | .[]? | select(._text == "tel:${phonenumber}")`,
        dentist.td, { input: 'json', output: 'json' }
    );

    expect(dentistName._text).to.eql(name)
    expect(dentistPhone._text).to.eql("tel:" + phonenumber)
});

// This fixture saves home community id, repository id and document unique id from a document
Then(
    'dokumentoplysningerne fra dokument {string} gemmes som {string}, {string} og {string}',
    async function (
        DKIHETypeCode,
        HomeCommunityID,
        RepositoryUniqueID,
        DocumentUniqueID
    ) {
        // Getting documents from last response
        let documentlist =
            this.getLastResponse().body.response.main.Envelope.Body
                .AdhocQueryResponse.RegistryObjectList.ExtrinsicObject;

        // Converting document type to type code
        const dkIHETypeCode = dkihetypecode.get(DKIHETypeCode);

        // Finding document with type code
        const document = await jq.run(
            `.[] | select(.Classification[] | .__nodeRepresentation == "${dkIHETypeCode}")`,
            documentlist,
            { input: 'json', output: 'json' }
        );

        // Saving the ids into variables
        const homeCommunityId = document.__home;
        const documentUniqueId = document.__id;
        const repositoryUniqueId = documentUniqueId.substring(
            0,
            documentUniqueId.indexOf('^')
        );

        // Setting the id's
        this.variables.set(HomeCommunityID, homeCommunityId);
        this.variables.set(RepositoryUniqueID, repositoryUniqueId);
        this.variables.set(DocumentUniqueID, documentUniqueId);

        // Expecting the document id no to be null
        expect(this.documentId).to.not.be.null;
        expect(this.documentHome).to.not.be.null;
    }
);

Then('dokumentoplysningerne fra dokument med formatcode {string} gemmes som {string}, {string} og {string}',
    async function (
        formatkode,
        HomeCommunityID,
        RepositoryUniqueID,
        DocumentUniqueID
    ) {
        // Getting documents from last response
        let documentlist =
            this.getLastResponse().body.response.main.Envelope.Body
                .AdhocQueryResponse.RegistryObjectList.ExtrinsicObject;

        // Finding document with type code
        const document = await jq.run(
            `.[] | select(.Classification[] | .__nodeRepresentation == "${formatkode}")`,
            documentlist,
            { input: 'json', output: 'json' }
        );

        // Saving the ids into variables
        const homeCommunityId = document.__home;
        const documentUniqueId = document.__id;
        const repositoryUniqueId = documentUniqueId.substring(
            0,
            documentUniqueId.indexOf('^')
        );

        // Setting the id's
        this.variables.set(HomeCommunityID, homeCommunityId);
        this.variables.set(RepositoryUniqueID, repositoryUniqueId);
        this.variables.set(DocumentUniqueID, documentUniqueId);

        // Expecting the document id no to be null
        expect(this.documentId).to.not.be.null;
        expect(this.documentHome).to.not.be.null;
    }
);

// This fixture expects the CDA document to contain doctor with name, yderNr and postal code
Then(
    'CDA indeholder læge med navn {string}, ydernr {string}, postnr {string} og telefonnummer {string}',
    function (providerName, providerNumber, postalCode, phonenumber) {
        let provider;
        // Getting health provider from last response
        if (this.getLastResponse().body.response.attachments[0] != null) {
            provider =
                this.getLastResponse().body.response.attachments[0].ClinicalDocument.recordTarget.patientRole.providerOrganization;
        } else { //Sometimes the response is not in the attachments
            provider =
                this.getLastResponse().body.response.main.Envelope.Body.RetrieveDocumentSetResponse.
                    DocumentResponse.Document._data.ClinicalDocument.recordTarget.patientRole.providerOrganization;
        }
        // Expecting name, yderNr and postal code from repsonse to equal name, yderNr and postal code from fixture
        expect(provider.name._text).to.eql(providerName);
        expect(provider.id.__extension).to.eql(providerNumber);
        expect(provider.addr.postalCode._text).to.eql(postalCode);
        expect(provider.telecom.__value).to.eql("tel:" + phonenumber)
    }
);

// This fixture counts the number of documents in the response and saves the amount
Then(
    'antallet af dokumenter i {string} gemmes som {string}',
    function (responseName, resultName) {
        // Getting response
        const response = this.variables.get(responseName);

        // Expecting response not to be null
        if (response == null) {
            throw new Error('Unable to find response: ' + responseName);
        }

        // Getting the documents from the response
        let documentlist =
            response.body.response.main.Envelope.Body.AdhocQueryResponse
                .RegistryObjectList.ExtrinsicObject;

        // If the documents are in an array
        if (Array.isArray(documentlist)) {
            // Setting result name to the number of documents
            this.variables.set(resultName, documentlist.length);

            // If the document list is undefined
        } else if (documentlist == undefined) {
            // Setting the result name to zero
            this.variables.set(resultName, 0);
        } else {
            // Setting the result name to one
            this.variables.set(resultName, 1);
        }
    }
);

// This fixture expects the respitory to contain warning
Then(
    'repository {string} indeholder en advarsel om {string}',
    function (ResponseName, Error) {
        // Getting warning
        const error =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .RetrieveDocumentSetResponse.RegistryResponse.RegistryErrorList
                .RegistryError.__codeContext;

        // Expecting the warning to be the same as in the fixture
        expect(error).to.eql(Error);
    }
);

// This fixture finds a document with a specific document id and pushes it
Then('dokumentet med {string} findes', async function (documentUniqueID) {
    // Getting documents from last response
    let documentlist =
        this.getLastResponse().body.response.main.Envelope.Body
            .AdhocQueryResponse.RegistryObjectList.ExtrinsicObject;

    // Replacing document unique id
    documentUniqueID = this.replaceVariables(documentUniqueID);

    let document
    // Finding document with type code
    if (Array.isArray(documentlist)) {
        document = await jq.run(
            `.[]  | select(.ExternalIdentifier[] | select(.__value == "${documentUniqueID}"))`,
            documentlist,
            { input: 'json', output: 'json' }
        );
    } else {
        document = documentlist
        expect(document.__id).to.eql(documentUniqueID);
    }

    // Attaching document
    // this.attach(JSON.stringify(document, null, 2), 'text/plain');
    // Pushing document
    this.documents.push(document);

    // Expecting the document to be not empty
    expect(document).to.not.eql("")
});

// This fixture expects a metadata field to exist
Then('metadatafelt {string} findes', async function (field) {
    // Getting last document
    const document = this.getLastDocument();

    // Getting array of booleans
    const element = await jq.run(`[.Slot[].__name == "${field}"]`, document, {
        input: 'json',
        output: 'json',
    });

    // Expects one of the elements in the array to be true
    expect(element.includes(true)).to.be.true;
});

// This fixture expects the patient id to have a given value
Then(
    'metadatafelt patient id {string} har værdien {string}',
    async function (field, value) {
        // Getting last document
        const document = this.getLastDocument();

        // Replacing variables
        value = this.replaceVariables(value);

        // Getting DK IHE identifier urn uuid
        const identifierId = dkiheidentifier.get(field);

        // Getting element with patient id
        const element = await jq.run(
            `.ExternalIdentifier[] | select(.__identificationScheme == "${identifierId}")`,
            document,
            { input: 'json', output: 'json' }
        );

        expect(element.__value).to.eql(value);
    }
);

// This fixture expects the source patient info to have 3 values
Then(
    'metadatafelt {string} har værdierne {string}, {string} og {string}',
    async function (field, value1, value2, value3) {
        // Getting last document
        const document = this.getLastDocument();

        // Getting element from slot array
        const element = await jq.run(
            `.Slot[]|select(.__name == "${field}")`,
            document,
            { input: 'json', output: 'json' }
        );

        expect(element.ValueList.Value[0]._text.substring(0, 5)).to.eql(value1);
        expect(element.ValueList.Value[1]._text.substring(0, 5)).to.eql(value2);
        expect(element.ValueList.Value[2]._text.substring(0, 5)).to.eql(value3);
    }
);

//Fixture specific for having a test case with stamkort, as we need to have two seperate documents for
//v2 and v3
Then(
    '{string} og {string} har metadatafelt {string} med værdien {string}',
    async function (document1, document2, dkiIdentifier, healthcareFacilityTypeCode) {
        document1 = this.variables.get(document1);
        document2 = this.variables.get(document2);

        // Getting DK IHE identifier urn uuid
        const identifierId = dkiheidentifier.get(dkiIdentifier);
        console.log(identifierId)

        // Getting element with patient id
        const element1 = await jq.run(
            `.Classification[] |select(.__classificationScheme=="${identifierId}")`,
            document1,
            { input: 'json', output: 'json' }
        );

        expect(element1.__nodeRepresentation).to.eql(healthcareFacilityTypeCode);

        //Flexible approach if we have a testcase with only one document
        if (document2 != undefined) {
            const element2 = await jq.run(
                `.Classification[] |select(.__classificationScheme=="${identifierId}")`,
                document2,
                { input: 'json', output: 'json' }
            );
            expect(element2.__nodeRepresentation).to.eql(healthcareFacilityTypeCode);
        }
    }
);

Then(
    'dokumenttype har metadatafelt {string} med værdien {string}',
    async function (dkiIdentifier, healthcareFacilityTypeCode) {
        //Getting last document
        const document = this.getLastDocument();

        // Getting DK IHE identifier urn uuid
        const identifierId = dkiheidentifier.get(dkiIdentifier);

        // Getting element with patient id
        const element = await jq.run(
            `.Classification[] |select(.__classificationScheme=="${identifierId}")`,
            document,
            { input: 'json', output: 'json' }
        );

        expect(element.__nodeRepresentation).to.eql(healthcareFacilityTypeCode);
    }
);

Then('{string} indeholder homeCommunityId {string}, repositoryUniqueId {string} og documentUniqueId {string}',
    async function (responseName, homeCommunityId, repositoryId, documentUniqueId) {
        let response = this.variables.get(responseName);

        let documentDetails = response.body.response.main.Envelope.Body.RetrieveDocumentSetResponse.DocumentResponse;

        expect(homeCommunityId).to.equal(documentDetails.HomeCommunityId._text);
        expect(repositoryId).to.equal(documentDetails.RepositoryUniqueId._text);
        expect(documentUniqueId).to.equal(documentDetails.DocumentUniqueId._text);

        expect(documentDetails.Document).not.to.equal(null);

        console.log(documentDetails.HomeCommunityId._text + "\n" + documentDetails.RepositoryUniqueId._text + "\n" + documentDetails.DocumentUniqueId._text)
    });


Then('{string} dokumentet har dokumentid {string}', async function (responseName, documentId) {
    let response = this.variables.get(responseName);
    let document = response.body.response.main.Envelope.Body.AdhocQueryResponse.RegistryObjectList.ExtrinsicObject

    if (Array.isArray(document)) {
        res = await jq.run(
            `.[]| .ExternalIdentifier[] | select(.__value == "${documentId}")`,
            document,
            { input: 'json', output: 'json' }
        );
    } else {
        res = await jq.run(
            `. | select(.ExternalIdentifier[]| select(.__value == "${documentId}"))`,
            document,
            { input: 'json', output: 'json' }
        );
    }
    expect(res.__value).to.equal(documentId);
});


////////////////////////\\\\\\\\\\\\\\\\\\\\\\\\\
//_______________________________________________\\
//--------NSP Request Response Transformation------\\
const superagent = require('superagent');
const _nrrt = {
    endpoint: 'https://testkald.nspop.dk/nrrt',
};

async function nrrtCall(header, inputstream, content) {
    const nrrtAgent = superagent
        .agent()
        .set('HeaderType', header)
        .set('Content-Type', content)

    const nrrt = function () {
        return nrrtAgent.post(_nrrt.endpoint);
    };
    let response;
    try {
        response = (await nrrt().send(inputstream));
    } catch (err) {
        throw err;
    }
    return response;
}

//sender XML-svaret til konvertering
Then('NRRT kaldes med HeaderType {string} og svaret gemmes som {string}', async function (HeaderType, nrrtResponse) {
    let input = this.getLastResponse().body.output;
    let header = this.replaceVariables(HeaderType);
    let content = this.getLastResponse().body.contentType;

    let response = await nrrtCall(header, input, content);
    this.variables.set(nrrtResponse, response);
    this.responses.push(response);
});

//Tæller antallet af DocumentEntries 
Then('{string} indeholder {int} documentEntry', function (nrrtresponse, amountOfEntries) {
    let resJson = JSON.parse(this.variables.get(nrrtresponse).text);
    let EntryAmount = amountOfEntries;

    let amount = resJson.DocumentEntries.length;
    expect(EntryAmount).to.equal(amount);
});

//Til at finde værdier til en specifik documentEntry
Then('{string} indeholder datasæt {string}', function (nrrtresponse, dataset) {
    const expectedDataset = this.datasets.get(dataset);
    const resJson = JSON.parse(this.variables.get(nrrtresponse).text).DocumentEntries[0];
    let response = resJson.DocumentEntry;

    let data = {
        'SourcePatientInfo': response.SourcePatientInfo.name,
        'SourcePatientId': response.SourcePatientId.Code,
        'AvailabilityStatus': response.AvailabilityStatus,
        'Type': response.Type,
        'TypeCode': response.TypeCode.name,
        'FormatCode': response.FormatCode.name,
        'authorPerson': response.Author[0].authorPerson.name,
        'authorInstitution': response.Author[0].authorInstitution[0].institution.name,
        'ClassCode': response.ClassCode.name,
        'ConfidentialCode': response.ConfidentialCode[0].name,
        'CreationTime': response.CreationTime,
        'ServiceStartTime': response.ServiceStartTime,
        'ServiceStopTime': response.serviceStopTime,
    }

    for (k of Object.entries(expectedDataset)) {
        try {
            expect(k[1]).to.equal(data[k[0]]);
        } catch (error) {
            console.log(k[0] + " var i stedet lig med " + data[k[0]]);
            throw error;
        }
    }
});

//Til at finde codeSchemes til en specifik documentEntry
Then('{string} indeholder codeSchemes {string}', function (nrrtresponse, dataset) {
    const expectedDataset = this.datasets.get(dataset);
    const resJson = JSON.parse(this.variables.get(nrrtresponse).text).DocumentEntries[0];
    let response = resJson.DocumentEntry;

    let data = {
        'SourcePatientId': response.SourcePatientId.codeScheme,
        'TypeCode': response.TypeCode.codeScheme,
        'FormatCode': response.FormatCode.codeScheme,
        'ClassCode': response.ClassCode.codeScheme,
        'ConfidentialCode': response.ConfidentialCode[0].codeScheme,
        'HealthcareFacility': response.HealthcareFacility.codeScheme,
        'PracticeSettingCode': response.PracticeSettingCode.codeScheme,
        'UniqueId': response.UniqueId
    }

    for (k of Object.entries(expectedDataset)) {
        try {
            expect(k[1]).to.equal(data[k[0]]);
        } catch (error) {
            console.log(k[0] + " var i stedet lig med " + data[k[0]]);
            throw error;
        }
    }
});

//For stamkort iti43 => ClincialDocument
Then('{string} indeholder stamkort med datasæt {string}', function (nrrtresponse, dataset) {
    const expectedDataset = this.datasets.get(dataset);
    const resJson = JSON.parse(this.variables.get(nrrtresponse).text);
    let response = resJson.ClinicalDocument;

    let name = "";
    response.nameAndAddress.personIdentity.givenNames.forEach(given => {
        name += " " + given.given;
    });

    let data = {
        'livingWillRegistration': response.livingWillRegistration.isRegistered,
        'organDonorRegistration': response.organDonorRegistration.isRegistered,
        'treatmentWillRegistration': response.treatmentWillRegistration.isRegistered,
        'givenNames': name.slice(1),
        'familyName': response.nameAndAddress.personIdentity.familyName,
        'streetAdressLine': response.nameAndAddress.addressData.streetLines[0].streetAddressLine,
        'postalCode': response.nameAndAddress.addressData.postalCode,
        'city': response.nameAndAddress.addressData.city,
        'country': response.nameAndAddress.addressData.country,
        'dentistAdress': response.dentist != undefined ? response.dentist.addressData.streetLines[0].streetAddressLine : '',
        'dentistPostcode': response.dentist != undefined ? response.dentist.addressData.postalCode : ''
    }

    for (k of Object.entries(expectedDataset)) {
        try {
            let actualResult = data[k[0]]
            if (actualResult == false) {
                expect(k[1]).to.equal('false')
            } else if (actualResult == true) {
                expect(k[1]).to.equal('true')
            } else {
                expect(k[1]).to.equal(actualResult);
            }
        } catch (error) {
            console.log(k[0] + " var i stedet lig med " + data[k[0]]);
            throw error;
        }
    }
});

Then('{string} indeholder organisationerne {string}', function (nrrtresponse, dataset) {
    const expectedDataset = this.datasets.get(dataset);
    const resJson = JSON.parse(this.variables.get(nrrtresponse).text);
    let response = resJson.ClinicalDocument;
    let data = {
        'livingWillRegistration': response.livingWillRegistration.authorOfInformation.representedOrganizationName,
        'organDonorRegistration': response.organDonorRegistration.authorOfInformation.representedOrganizationName,
        'treatmentWillRegistration': response.treatmentWillRegistration.authorOfInformation.representedOrganizationName,
        'coverage': response.coverage.authorOfInformation.representedOrganizationName,
        'nameAndAddress': response.nameAndAddress.authorOfInformation.representedOrganizationName,
    }

    for (k of Object.entries(expectedDataset)) {
        try {
            expect(k[1]).to.equal(data[k[0]]);
        } catch (error) {
            console.log(k[0] + " var i stedet lig med " + data[k[0]]);
            throw error;
        }
    }
});

Then('registersvar indeholder ikke dokument med dokumenttype {string}', async function (documenttype) {
    // Getting the DK IHE Type Code by the dokument type
    const typeValue = dkihetypecode.get(documenttype);

    // This urn is the IHE identifier for typecode. NSP uses this to identify document type
    // https://gazelle.ihe.net/XDStarClient/rimihe-templates/rimihe-TypeCode_XDSDocumentEntry.html
    const typeCode = 'urn:ad:dk:medcom:pdc-v3.0:full';

    // Getting the documents from the last response
    let documentlist =
        this.getLastResponse().body.response.main.Envelope.Body
            .AdhocQueryResponse.RegistryObjectList.ExtrinsicObject;

    // Making result variable
    let res;
    // If the documents is stored in an array
    if (Array.isArray(documentlist)) {
        // Finding the document with the right type code and type value
        res = await jq.run(
            // Selecting document from an array by type code and value
            `.[] | select(.Classification[] | .__classificationScheme == "${typeCode}" and .__nodeRepresentation == "${typeValue}")`,
            documentlist,
            { input: 'json', output: 'json' }
        );
    } else {
        // Finding the document with the right type code and type value
        res = await jq.run(
            // Selecting document by type code and value
            `. | select(.Classification[] | .__classificationScheme == "${typeCode}" and .__nodeRepresentation == "${typeValue}")`,
            documentlist,
            { input: 'json', output: 'json' }
        );
    }

    // Expecting the document to exist and not be equal to nothing
    expect(res, 'Document must exist').to.eql('');
});

Then('der findes ikke {string} med formattype {string} dokumentet har id {string}', async function (documenttype, formattype, documentid) {
    let id = '1.2.208.176.43210.8.10.12^babbdc73-f52a-449c-a61f-30032914b27a';
    const type = formattype;

    let documentlist =
            this.getLastResponse().body.response.main.Envelope.Body
                .AdhocQueryResponse.RegistryObjectList.ExtrinsicObject;

    let res;
    if (Array.isArray(documentlist)) {
        // Finding the document with the right type code and type value
        res = await jq.run(
        // Selecting document from an array by type code and value
        `.[] | select(.Classification[] |.__nodeRepresentation == "${type}")`,
        documentlist,
        { input: 'json', output: 'json' }
        );
    } else {
        // Finding the document with the right type code and type value
        res = await jq.run(
        // Selecting document by type code and value
        `. | select(.Classification[] |.__nodeRepresentation == "${type}")`,
        documentlist,
        { input: 'json', output: 'json' }
        );
    }
       expect(id).to.eql(documentid);
    }
);
