const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const { expect } = require('../util');
const { getPersonsList, getPatientDetails } = require('dataaccess/dtg');
const jq = require('node-jq');

When('DTG kaldes med søgefelt {string} og svar gemmes som {string}', async function (searchWord, persons) {
    let response = await getPersonsList(this.attach)

    searchWord = this.replaceVariables(searchWord)
    this.responses.push(response);

    const sw = searchWord === "NULL" ? "" : searchWord.trim().toLowerCase();
    const searchWords = sw.split(" ")

    let results = response.body.persons.filter(p => {
        const fields = ["fornavn", "mellemnavn", "efternavn", "entity_id"]
            .map(key => (p[key] || "").toLowerCase())
            .join(" ");

        if (searchWords.length === 1) {
            return fields.includes(searchWords[0]);
        } else {
            return searchWords.every(word => fields.includes(word));
        }
    });
    this.variables.set(persons, results);
});

When('persondetaljer for borger {string} hentes og gemmes som {string}', async function (entity_id, response) {
    entity_id = this.replaceVariables(entity_id)
    let patientDetails = await getPatientDetails(this.attach, entity_id);

    this.responses.push(patientDetails);
    this.variables.set(response, patientDetails.body);
});


Then('DTG service svarer {string}', function (status) {
    response = this.getLastResponse()
    if (status == "ok") {
        expect(response.statusCode).to.eql(200)
    } else {
        expect(response.statusCode).to.eql(500)
    }
});

Then('antallet af DTG borgere i {string} er {string} {int}', function (response, state, amount) {
    persons = this.replaceVariables(response)
    switch (state) {
        case "større end":
            expect(persons.length).to.greaterThan(amount)
            break;
        case "mindre end":
            expect(persons.length).to.lessThan(amount)
            break;
        case "lig med":
            expect(persons.length).to.eql(amount)
            break;
        default:
            throw new Error(`Unknown comparison "${state}"`);
    }
});

Then('DTG borger {string} findes i borgerlisten {string}', function (entity_id, response) {
    persons = this.replaceVariables(response)
    entity_id = this.replaceVariables(entity_id)

    let person = persons.filter(p =>
        ["entity_id"].some(
            key => (p[key]).includes(entity_id)
        )
    );
    expect(person.length).to.eql(1)
    expect(person[0].entity_id).to.eql(entity_id)
});

Then('{string} indeholder persondetaljerne {string}', function (response, dataset) {
    const expectedDataset = this.datasets.get(dataset);
    response = this.replaceVariables(response);

    for (const [key, expectedValue] of Object.entries(expectedDataset)) {
        const actualValue = response.person[key];
        if (expectedValue === '' && actualValue === undefined) {
            continue;
        }
        expect(String(expectedValue)).to.equal(String(actualValue));
    }
});

Then('{string} indeholder {string} {string}', async function (response, info, expectedValue) {
    response = this.replaceVariables(response);
    switch (info) {
        case 'event':
            res = await jq.run(
                `.events[] | select(.description == "${expectedValue}")`,
                response,
                { input: 'json', output: 'json' }
            );
            expect(res.description).to.eql(expectedValue)
            break;
        case 'autorisation':
            res = await jq.run(
                `.autorisationer[] | select(.autorisationsId == "${expectedValue}")`,
                response,
                { input: 'json', output: 'json' }
            );
            expect(res.autorisationsId).to.eql(expectedValue)
            break;
        default:
            throw new Error('Not recognized ' + info);
    }
});