const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const { GetConsent, GetConsentCustody, DeleteConsent, personLookup, RegisterConsent, RegisterConsentCustody, RegisterConsentAdmin, DeleteConsentAdmin } = require('dataaccess/fgvhr');
const jq = require('node-jq');
const { expect, parseRelativeDate } = require('../util');
const roles = new Map(require('../../config/roles.json'));


///////////////////////////////////////////////////////////////
//              Given statements                             //
///////////////////////////////////////////////////////////////

//tjek på om borger har en værge
Given('borger {string} er værge til borger {string}',
    async function (guardianCPR, patientCpr) {
        patientCpr = this.replaceVariables(patientCpr);

        const response = await personLookup(this.attach, patientCpr)
        const guardian =
            response.body.response.main.Envelope.Body.getPersonInformationOut
                .PersonInformationStructure.PersonCustodyStructure;

        let responseGuardian;
        if (guardian[1] != null) {
            guardian.forEach(custody => {
                if (custody.PersonCustodyRelation._text == 'vaerge') {
                    responseGuardian = custody.PersonCustodyIdentifier._text;
                }
            });
        } else {
            if (guardian.PersonCustodyRelation._text == 'vaerge') {
                responseGuardian = guardian.PersonCustodyIdentifier._text;
            }
        }
        expect(guardianCPR).to.eql(responseGuardian);
    });

//checker at borger er under (eksempelvis 60)
Given('borger {string} alder er {string} {int}', async function (patientCpr, status, age) {
    patientCpr = this.replaceVariables(patientCpr);
    var responseAge;
    const response = await personLookup(this.attach, patientCpr)
    const responseBirth =
        response.body.response.main.Envelope.Body.getPersonInformationOut
            .PersonInformationStructure.RegularCPRPerson.PersonBirthDateStructure.BirthDate._text;

    let part = responseBirth.slice(0, 10);
    let date = new Date(part);

    if (new Date().getFullYear() - date.getFullYear() == 60) {
        if (new Date().getMonth() < date.getMonth()) {
            responseAge = new Date().getFullYear() - date.getFullYear() - 1;
        }
        else if (new Date().getMonth() == date.getMonth()) {
            if (new Date().getDate() < date.getDate()) {
                responseAge = new Date().getFullYear() - date.getFullYear() - 1;
            }
            else {
                responseAge = new Date().getFullYear() - date.getFullYear();
            }
        }
        else {
            responseAge = new Date().getFullYear() - date.getFullYear();
        }
    } else {
        responseAge = new Date().getFullYear() - date.getFullYear();
    }
    console.log(responseAge);
    switch (status) {
        case 'under':
            expect(responseAge).to.lessThan(age);
            break;
        case 'over':
            expect(responseAge).to.greaterThanOrEqual(age);
            break;
    }
});

//sikre at borgeren har en fravalgsregistrering med status draft
Given('fravalgsregistrering er oprettet for {string} af medarbejder',
    { timeout: -1 },
    async function (patientCpr) {
        patientId = this.replaceVariables(patientCpr);
        let today = new Date();
        var datetime = today.getFullYear() + "-" + (today.getMonth() + 1) + "-" + today.getDate();
        await RegisterConsentAdmin(this.attach, patientId, today);
    });

//sikre at borgeren har en inaktiv fravalgs registrering lavet af admin
Given('fravalgsregistrering er slettet for {string} af medarbejder med {string}',
    async function (patientCpr, date) {
        patientId = this.replaceVariables(patientCpr);
        date = this.replaceVariables(date);
        await DeleteConsentAdmin(this.attach, patientId, date);
    });

//sikre at borgeren har en fravalgsregistrering med status draft
Given('fravalgsregistrering er oprettet for {string} af borger',
    async function (patientCpr) {
        patientId = this.replaceVariables(patientCpr);
        await RegisterConsent(this.attach, patientId);
    });



// sikre at borgeren har en inaktiv fravalgs registrering    
Given('fravalgsregistrering er slettet for {string} af borger',
    async function (patientCpr) {
        patientId = this.replaceVariables(patientCpr);
        await DeleteConsent(this.attach, patientId);
    });

Given('{string} registrerer fravalg af genoplivning for borger {string}',
{ timeout: -1 },
    async function (custodyCpr, patientCpr) {
        patientId = this.replaceVariables(patientCpr);
        custodyId = this.replaceVariables(custodyCpr);


        const response = await RegisterConsentCustody(this.attach, custodyId, patientId);
        this.responses.push(response);

    });

Given('{string} henter fravalg af genoplivning for borger {string}', async function (custodyCpr, patientCpr) {
    patientId = this.replaceVariables(patientCpr);
    custodyId = this.replaceVariables(custodyCpr);
    console.log("this is P", patientId, "and c", custodyId);

    const response = await GetConsentCustody(this.attach, custodyId, patientId);
    this.responses.push(response);

});

// Tjekker om hvilken type FGVHR registrering der findes for borgeren 
Given('der eksisterer en FGVHR registrering for borger {string} med status {string}',
    async function (patientCpr, status) {
        patientId = this.replaceVariables(patientCpr);

        value = await GetConsent(this.attach, patientId);
        expect(value).to.eql(status);
    });

///////////////////////////////////////////////////////////////
//              Then statements                             //
///////////////////////////////////////////////////////////////

Then('Oprettet dato for registreringen er {string} i {string}',
    async function (createdDate, responseName) {
        let response = this.variables.get(responseName);
        createdDate = this.replaceVariables(createdDate);
        createdDate = createdDate.slice(0,16);
        let createdDateInResponse = response.body.response.main.Envelope.Header.Security.Timestamp.Created._text
        let dateResponse = createdDateInResponse.slice(0,16);
        expect(dateResponse).to.eql(createdDate)
    });

Then('{string} gemmes som {string}', function (pastDate, date) {
    createdDate = parseRelativeDate(pastDate)

    createdDate = createdDate.slice(0, 10);

    this.variables.set(date, createdDate);
});

//Tjekker dato
Then('{string} er {string} i {string}', function (dateAttribute, datetime, response) {
    let fgvhrResponse = this.variables.get(response);
    let date = this.replaceVariables(datetime);
    let createdDateInResponse;
    switch (dateAttribute) {
        case 'oprettelsesdato':
        case 'slettedato':
            createdDateInResponse = fgvhrResponse.body.response.main.Envelope.Body.Consent.date.__value;
            expect(createdDateInResponse).to.eql(date)
            break;
        default:
            expect(createdDateInResponse).to.eql(date)
            break;
    }
});


// Tjekker om hvilken type FGVHR registrering der findes for borgeren i et kald
Then('der {string} en FGVHR registrering for borger med cpr {string} med status {string} i {string}',
    async function (status, patientCpr, registrering, responseName) {
        let response = this.variables.get(responseName);
        patientCpr = this.replaceVariables(patientCpr);
        registrering = this.replaceVariables(registrering);

        switch (status) {
            case 'findes':
                let registreringsStatus = response.body.response.main.Envelope.Body.Consent.status.__value
                console.log(registreringsStatus);
                expect(registreringsStatus).to.eql(registrering)

                let patientCprISvar =
                    response.body.response.main.Envelope.Body.Consent.grantor.identifier.value.__value
                console.log(patientCprISvar);
                expect(patientCprISvar).to.eql(patientCpr)
                break

            case 'findes ikke':
                console.log("findes ikke");
                break
            default:
                break
        }
    });

When('fravalgs registreringen for {string} slettes', async function (patientCpr) {
    patientId = this.replaceVariables(patientCpr);
    response = await DeleteConsent(this.attach, patientId);
});



