const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const { expect, parseRelativeDate, findPathsAndValues } = require('../util');
const jq = require('node-jq');
const {
    getConsentRegistration,
    createConsentRegistration,
} = require('dataaccess/minspaerring');

Given(
    'der findes en spærring for {string} med datasæt {string}',
    { timeout: -1 },
    async function (PatientCPR, datasetName, dataTable) {
        PatientCPR = this.replaceVariables(PatientCPR);

        const data = this.parseDataTable(dataTable);
        this.datasets.set(datasetName, data);

        const consentRegistrations = await getConsentRegistration(
            this.attach,
            PatientCPR
        );
        if (Array.isArray(consentRegistrations)) {
            const matches = consentRegistrations.filter((consent) => {
                const flat = findPathsAndValues(consent);
                return Object.entries(data).reduce((res, entry) => {
                    const actual = flat['.' + entry[0] + '._text'];
                    const expected = entry[1];
                    // ignore matching empty string
                    if (expected == '') {
                        return res;
                    }
                    return res && actual == expected;
                }, true);
            });
        } else {
            const flat = findPathsAndValues(consentRegistrations);
            return Object.entries(data).reduce((res, entry) => {
                const actual = flat['.' + entry[0] + '._text'];
                const expected = entry[1];
                // ignore matching empty string
                if (expected == '') {
                    return res;
                }
                return res && actual == expected;
            }, true);
        }
    }
);

Given(
    'antallet af spærringer for {string} gemmes som {string}',
    async function (PatientCPR, variable) {
        // Funktion slår op i minspaerring.administration.ConsentRegistrationsGet
        // og tæller antal ConsentRegistrations og gemmer dem under variable med det angivne navn
        PatientCPR = this.replaceVariables(PatientCPR);

        const consentRegistrations = await getConsentRegistration(
            this.attach,
            PatientCPR
        );
        console.log(consentRegistrations);
        if (consentRegistrations == undefined) {
            //Zero Consent Registrations
            this.variables.set(variable, 0);
        } else if (consentRegistrations.length != 0) {
            //Several Consent Registrations
            this.variables.set(variable, consentRegistrations.length);
            this.documents.push(
                consentRegistrations[consentRegistrations.length - 1]
            );
        } else {
            //One Consent Registration
            this.variables.set(variable, 1);
            this.documents.push(consentRegistrations);
        }
    }
);

Then(
    'nyeste spærring for {string} har datasæt {string}',
    function (PatientCPR, DataSaet) {
        // Opslag i minspaerring.administration.ConsentRegistrationsGet, find seneste objekt og sammenlign med værdier fra datasæt
        const dataset = this.datasets.get(DataSaet);
        console.log("this is dataset", dataset);

        const consent = this.getLastDocument();
        console.log("this is consent", consent);

        //Citizen CPR
        expect(consent.citizenCPR._text).to.eql(
            PatientCPR && dataset.CitizenCivilRegistrationNumber
        );

        //Valid From Date
        const validFromDate = new Date(consent.validFromDate._text);

        expect(validFromDate.toISOString().split('.')[0] + 'Z').to.eql(
            dataset.validFromDate
        );

        //Who Or What Item
        if (dataset.item == 'whoitem') {
            if (dataset.who == 'positiveConsent') {
                //Positive Consent
                expect(consent.positiveConsent._text).to.eql(
                    dataset.positiveConsent
                );
            }
            else if (dataset.who == 'healthProfessionalCPR') {

                //Health Professional CPR
                expect(consent.who.healthProfessionalCPR._text).to.eql(
                    dataset.healthProfessionalCPR
                );

            }
            else if (dataset.who == 'foreignHealthProfessionals') {
                expect(consent.who.foreignHealthProfessionals._text).to.eql(
                    dataset.foreignHealthProfessionals)
            }
            else if (dataset.who == 'organizationIdentifier') {
                expect(consent.who.organizationIdentifier._text).to.eql(
                    dataset.organizationIdentifier)
            }

            //Contains Who Item
            expect(consent).to.include.any.keys('who');
        } else if (dataset.item == 'whatitem') {
            //Organization Identifier
            expect(consent.what.organizationIdentifier._text);

            //Include Sub Organizations
            expect(consent.what.includeSubOrganizations._text);

            //Referral Start Date
            const referralStartDate = dataset.ReferralStartDate;

            expect(consent.what.referralStart._text).to.eql(
                referralStartDate.split('.')[0] +
                '+' +
                referralStartDate.split('+')[1]
            );

            //Referral End Date
            const referralEndDate = dataset.ReferralEndDate;

            expect(consent.what.referralEnd._text).to.eql(
                referralEndDate.split('.')[0] +
                '+' +
                referralStartDate.split('+')[1]
            );

            //Contains What Item
            expect(consent).to.include.any.keys('what');
        }
    }
);

Then(
    'der {string} en aftale for {string} hos {string} på dato {string} i {string}',
    async function (Status, PatientCPR, SORCode, Date, ResponseName) {
        PatientCPR = this.replaceVariables(PatientCPR);
        SORCode = this.replaceVariables(SORCode);

        let response = this.variables.get(ResponseName);

        let extrinsicObjects =
            response.body.response.main.Envelope.Body.AdhocQueryResponse
                .RegistryObjectList.ExtrinsicObject;

        //Finder aftale dokument for PatientCPR
        let object = await jq.run(
            `[.[]|select(.Name.LocalizedString.__value|startswith("Aftale for ${PatientCPR}"))]`,
            extrinsicObjects,
            { input: 'json', output: 'json' }
        );
        console.log(object)
        //Checking for correct date
        let objectDate = await jq.run(
            `[.[]|select(.Slot[]? |select((.__name=="serviceStartTime") and (.ValueList.Value._text|startswith("${Date}"))))]`,
            object,
            { input: 'json', output: 'json' }
        );

        //Checking for correct classification
        let classification = await jq.run(
            `[.[].Classification[]|select(.__classificationScheme=="urn:uuid:93606bcf-9494-43ec-9b4e-a7748d1a838d")]`,
            objectDate,
            { input: 'json', output: 'json' }
        );

        var sorCode;
        stringified = JSON.stringify(classification, null, 2)

        //Some of the tests have multiple objects in the "Slot"-section in the response, making it an array
        //So this is a way to filter out which one does, so we can select the right query
        if (stringified.includes("Slot\": [")) {
            sorCode = await jq.run(
                `.[]|select(.Slot[]|select(.ValueList.Value._text|endswith("${SORCode}")))`,
                classification,
                { input: 'json', output: 'json' }
            );
        } else {
            sorCode = await jq.run(
                `.[]|select(.Slot|select(.ValueList.Value._text|endswith("${SORCode}")))`,
                classification,
                { input: 'json', output: 'json' }
            );
        }

        switch (Status) {
            case 'findes':
                expect(sorCode).not.to.eql('');
                break;
            case 'findes ikke':
                expect(sorCode).to.eql('');
                break;
            default:
                break;
        }
    }
);

Then(
    'registersvar {string} indeholder en advarsel om {string}',
    async function (ResponseName, Error) {
        let response =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .AdhocQueryResponse.RegistryErrorList.RegistryError;
        if (response.length > 0) {
            const error = await jq.run(
                `.[] | select(.__codeContext == "urn:dk:nsi:${Error}")`,
                response,
                { input: 'json', output: 'json' }
            );
            expect(error.__codeContext).to.eql('urn:dk:nsi:' + Error);
        } else {
            expect(response.__codeContext).to.eql('urn:dk:nsi:' + Error);
        }
    }
);

Then(
    'der {string} dokumenter i register for {string}',
    function (Status, ResponseName) {
        let response =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .AdhocQueryResponse.RegistryObjectList.ExtrinsicObject;

        switch (Status) {
            case 'findes':
                expect(response).not.to.eql(0);
                break;
            case 'findes ingen':
                expect(response).to.eql(undefined);
                break;
            default:
                break;
        }
    }
);

Then('{string} er lig {int}', function (Documents, Ten) {
    Documents = this.replaceVariables(Documents);
    expect(Documents).to.eql(Ten);
});

Given(
    'der findes {int} spærringer på {string}',
    async function (Amount, PatientCPR) {
        PatientCPR = this.replaceVariables(PatientCPR);
        let consentRegistrations = await getConsentRegistration(
            this.attach,
            PatientCPR
        );

        if (consentRegistrations == undefined) {
            let amount = 0;
            expect(amount).to.eql(Amount);
        } else if (consentRegistrations.length != undefined) {
            let count = 0;
            consentRegistrations.forEach(element => {
                if (element.validToDate != null) {
                    count++;
                    console.log(count);
                }
            })
            let amount = consentRegistrations.length - count;
            expect(amount).to.eql(Amount);
        } else {
            let amount = 1;
            expect(amount).to.eql(Amount);
        }
    }
);

Given(
    'der findes en aktiv spærring for {string} og id gemmes som {string}',
    async function (PatientCPR, ConsentId) {
        PatientCPR = this.replaceVariables(PatientCPR);

        let consentRegistrations = await getConsentRegistration(
            this.attach,
            PatientCPR
        );

        let consentRegistrationsWithOutValidToDate = await jq.run(
            `[.[]|select(.validToDate == null)]`,
            consentRegistrations,
            { input: 'json', output: 'json' }
        );
        this.variables.set(
            ConsentId,
            consentRegistrationsWithOutValidToDate[0].consentId._text
        );
    }
);

Then(
    'spærring for {string} med {string} har slutdato lig med {string}',
    async function (PatientCPR, ConsentId, EndDate) {
        PatientCPR = this.replaceVariables(PatientCPR);
        ConsentId = this.replaceVariables(ConsentId);

        let endDate = parseRelativeDate(EndDate).substring(0, 10);

        let consentRegistrations = await getConsentRegistration(
            this.attach,
            PatientCPR
        );

        let consentRegistration = await jq.run(
            `.[]|select(.consentId._text=="${ConsentId}")`,
            consentRegistrations,
            { input: 'json', output: 'json' }
        );

        expect(
            consentRegistration.validToDate._text.substring(0, 10)
        ).not.to.eql(undefined);
        expect(consentRegistration.validToDate._text.substring(0, 10)).to.eql(
            endDate
        );
    }
);

Given('min spærring er oprettet for {string}', async function (PatientCPR) {
    PatientCPR = this.replaceVariables(PatientCPR);
    await createConsentRegistration(this.attach, PatientCPR);
});

Then('frabelse er slettet for {string}', async function (PatientCPR) {
    PatientCPR = this.replaceVariables(PatientCPR);
    await ConsentRegistration(this.attach, PatientCPR);
});

Then('bruger samtykke verifikation er {string}', function (verificationResult) {
    // Write code here that turns the phrase above into concrete actions
    verificationResult = this.replaceVariables(verificationResult);
    let response = this.getLastResponse().body.response.main.Envelope.Body;
    console.log(response.ConsentForUserCheckResponse);
    expect(response.ConsentForUserCheckResponse._text).to.eql(verificationResult);
});

Then('data samtykke verifikation indeholder {string}', function (dokumentId) {
    // Write code here that turns the phrase above into concrete actions
    dokumentId = this.replaceVariables(dokumentId);
    let response = this.getLastResponse().body.response.main.Envelope.Body.PositiveConsentDataRegistrations;
    console.log(response.DataIdentifiers);   
    if (Object.keys(response.DataIdentifiers).length === 0) {
         throw new Error(`DokumentetId "${dokumentId}" findes ikke i svar:"${JSON.stringify(response.DataIdentifiers,null,2)}"`)
    } else {   
        expect(JSON.stringify(response.DataIdentifiers,null,2)).contains(dokumentId);
    };   
});

Then('data samtykke verifikation indeholder ikke {string}', function (dokumentId) {
    // Write code here that turns the phrase above into concrete actions
    dokumentId = this.replaceVariables(dokumentId);
    let response = this.getLastResponse().body.response.main.Envelope.Body.PositiveConsentDataRegistrations;
    console.log(response.DataIdentifiers);   
    if (Object.keys(response.DataIdentifiers).length === 0) {
        return true;
    } else {
           expect(JSON.stringify(response.DataIdentifiers,null,2)).not.contains(dokumentId);  
        };    
});