const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const jq = require('node-jq');
const { v4 } = require('uuid');
const { expect, parseRelativeDate } = require('../util');
const {
    destroyNAS,
    createNAS,
    deleteNASIDList,
    createNASIDList,
    unsubscribe,
} = require('../../dataaccess/nas');
const {
    CreateTemporaryAddress,
    DeleteTemporaryAddress,
    UpdateTemporaryAddress,
    CreateHealthProvider,
    DeleteHealthProvider,
    UpdateHealthProvider,
    CreateLanguage,
    DeleteLanguage,
    UpdateLanguage,
    CreateRelative,
    DeleteRelatives,
    UpdateRelatives,
    GetPersonalDataCard,
    UpdateContactInformation,
} = require('../../dataaccess/skr');
const {
    CreateTreatmentWill,
    UpdateTreatmentWillAdmin,
    DeleteTreatmentWill,
} = require('../../dataaccess/btr');
const { CreateOrganDonorRegistration,
    DeleteOrganDonorRegistration,
} = require('../../dataaccess/odr');
const { getPDCEntries } = require('../../dataaccess/sfsk');

//#region PullPoint fixtures
// This fixture saves a pull point in a variable
Then('pullpoint gemmes som {string}', function (ResultName) {
    // Saving a pull point from the last response
    const pullPoint =
        this.getLastResponse().body.response.main.Envelope.Body
            .CreatePullPointResponse.PullPoint.Address._text;

    // Setting pull point (result name) to the pull point found in the response
    this.variables.set(ResultName, pullPoint);
});

// This fixture deletes a pull point
Then(
    'pullpoint {string} slettes',
    // It is an async function because await is used
    async function (PullPoint) {
        // Getting pull point
        const pullPoint = this.variables.get(PullPoint);

        // Destroying pull point with NAS Destroy Pull Point service
        await destroyNAS(this.attach, pullPoint);
    }
);

// This fixture creates a pull point and saves it
Given(
    'der oprettes et pullpoint og gemmes som {string}',
    // It is an async function because await is used
    async function (PullPoint) {
        // Creating a pull point and saving the response
        const nas = await createNAS(this.attach);

        // Setting pull point to the pull point found in the response
        this.variables.set(
            PullPoint,
            nas.body.response.main.Envelope.Body.CreatePullPointResponse
                .PullPoint.Address._text
        );
    }
);

// This fixture expects a response to contain message about 'ExistingListUpdated'
Then(
    'retursvar indeholder ExistingListUpdated lig med {string}',
    function (ExistingListUpdated) {
        // Getting existing list updated boolean from the last response
        const exitstingListUpdated =
            this.getLastResponse().body.response.main.Envelope.Body
                .CreateIDListResponse.ExistingListUpdated._text;

        // Expecting the boolean from the response to equal the value from the fixture
        expect(exitstingListUpdated).to.eql(ExistingListUpdated);
    }
);
//#endregion

//#region ID List fixtures
// This fixture expects an id list to be deleted after having deleted it
Then(
    'idliste {string} slettes',
    // It is an async function because await is used
    async function (IDListName) {
        // Deleting an id list by its name and saving the response
        const result = await deleteNASIDList(this.attach, IDListName);

        // Expecting the result field 'NoSuchList' to equal false
        expect(
            result.body.response.main.Envelope.Body.DestroyIDListResponse
                .NoSuchList._text
        ).to.eql('false');
    }
);

// This fixture creates an id list with a given data table
Given(
    'der oprettes en idliste med datasæt',
    // It is an async function because await is used
    async function (dataTable) {
        // Parsing the data table and saving it
        const data = this.parseDataTable(dataTable);

        // Creating an id list by attaching the information from the data table
        await createNASIDList(
            this.attach,
            data.IDListName,
            data.Description,
            data.IDType,
            data.IDList
        );
    }
);
//#endregion

//#region Subscription fixtures
// This fixture saves a subscription
Then('subscription gemmes som {string}', function (SubscriptionRef) {
    // Getting the subscription reference from last response (NAS Subscribe)
    const subscription =
        this.getLastResponse().body.response.main.Envelope.Body
            .SubscribeResponse.SubscriptionReference.Address._text;

    // Setting subscription reference variable to subscription found in the response
    this.variables.set(SubscriptionRef, subscription);
});

// This fixture removes a subscription
Then(
    'subscription med {string} fjernes igen',
    // It is an async function because await is used
    async function (SubscriptionRef) {
        // Getting subscription reference
        const subscription = this.variables.get(SubscriptionRef);

        // Unsubscribing by attaching the subscription and calling NAS Unsubscribe service
        await unsubscribe(this.attach, subscription);
    }
);
//#endregion

//#region Notification fixtures
Then('der findes en notifikation for {string} som gemmes som {string}', async function (BTR, Notification) {
    const response =
        this.getLastResponse().body.response.main.Envelope.Body
            .GetMessagesResponse.NotificationMessage;

    this.variables.set(Notification, response);
    this.variables.set(Notification, response);
    console.log(response);

    this.variables.set(Notification, response);
    console.log(response);

});


// This fixture finds a notification with the given job id
Then(
    'der findes en notifikation for jobid {string} som gemmes som {string}',
    // It is an async function because await is used
    async function (JobId, Notification) {
        // Getting the last saved response and narrowing it to the notification messages
        const response =
            this.getLastResponse().body.response.main.Envelope.Body
                .GetMessagesResponse.NotificationMessage;

        // Converting to Job Id
        JobId = this.variables.get(JobId);

        // Finding notification with the given job id
        let notification = await jq.run(
            `.|select(.Message.NotifyContent.JobStatusNotification.JobId._text="${JobId}")`,
            response,
            { input: 'json', output: 'json' }
        );

        // Setting the notification variable to the notification found in the response
        this.variables.set(Notification, notification);
    }
);

// This fixture expects a notification to have a field with a given value
Then(
    '{string} har {string} lig med {string}',
    async function (Notification, Field, Value) {
        // Getting the notification from previous fixture
        let notification = this.variables.get(Notification);

        // Giving the field the right formalia
        const field = '__' + Field;

        // Converting to value (could be CPR, ?, http, OK etc.)
        const value = this.replaceVariables(Value);

        if (notification[0] != null) {
            notification = await jq.run(
                `.[1]`,
                notification,
                { input: 'json', output: 'json' }
            );
        }
        expect(notification.Message.NotifyContent[field]).to.eql(value);
    }
);

// This fixtur expects a notification with the field 'Topic' to equal a given topic value
Then('{string} har Topic lig med {string}',
    async function (Notification, Topic) {
        console.log("1:this is Notification", Notification);
        // Getting the notification from previous fixture
        let notification = this.variables.get(Notification);
        console.log("2:This is notification", notification);
        console.log("3:her er topic i notification", notification.Topic);
        // Expecting topic value in the notification to equal the given value
        if (notification[0] != null) {
            notification = await jq.run(
                `.[1]`,
                notification,
                { input: 'json', output: 'json' }
            );
        }
        expect(notification.Topic._text).to.eql(Topic);
    }
);

// This fixture expects a notification with the field 'JobStatus' to equal a given JobStatus value
Then(
    '{string} har JobStatus lig med {string}',
    function (Notification, JobStatus) {
        // Getting the notification from previous fixture
        const notification = this.variables.get(Notification);

        // Expecting job status value in the notification to equal the given value
        expect(
            notification.Message.NotifyContent.JobStatusNotification.JobStatus
                ._text
        ).to.eql(JobStatus);
    }
);

// This fixture finds and saves a notification with the given data-card-updated-message-id
Then(
    'der findes en notifikation for messageId {string} som gemmes som {string}',
    // It is an async function because await is used
    async function (MessageId, Notification) {
        // Getting the notification messages from the last response
        const response =
            this.getLastResponse().body.response.main.Envelope.Body
                .GetMessagesResponse.NotificationMessage;

        // Getting the message id from a previous fixture
        MessageId = this.variables.get(MessageId);
        console.log("this is messageid", MessageId);

        // Making notification variable with no value
        let notification;

        // If the response is an array
        if (Array.isArray(response)) {
            console.log(1);
            // Finding the notifications where the data card has been updated and selecting the one with the right message id.
            console.log(response);
            notification = await jq.run(
                `.[]|select(.Message.NotifyContent.DataCardUpdated.messageId.__value=="${MessageId}")`,
                response,
                { input: 'json', output: 'json' }
            );
            console.log(notification);
        } else {
            console.log(2);
            // If the response is not an array
            // Finding the notifications where the data card has been updated and selecting the one with the right message id.
            notification = await jq.run(
                `.|select(.Message.NotifyContent.DataCardUpdated.messageId.__value=="${MessageId}")`,
                response,
                { input: 'json', output: 'json' }
            );
        }
        console.log("this is notifi", notification, "this is Not", Notification);

        // Setting the notification variable to the notification found in the response
        this.variables.set(Notification, notification);
    }
);

Then('der findes en notifikation for borger {string} som gemmes som {string}',
    async function (PatientCPR, Notification) {
        // Getting the notification messages from the last response
        const response =
            this.getLastResponse().body.response.main.Envelope.Body
                .GetMessagesResponse.NotificationMessage;

        // Getting the CPR from a previous fixture
        PatientCPR = this.variables.get(PatientCPR);

        // Making notification variable with no value
        let notification;

        // If the response is an array
        if (Array.isArray(response)) {
            // Finding the notifications where the data card has been updated and selecting the one with the right CPR
            notification = await jq.run(
                `.[]|select(.Message.NotifyContent.__id=="${PatientCPR}")`,
                response,
                { input: 'json', output: 'json' }
            );
            console.log(notification);
        } else {
            // If the response is not an array
            // Finding the notifications where the data card has been updated and selecting the one with the right CPR
            notification = await jq.run(
                `.|select(.Message.NotifyContent.__id=="${PatientCPR}")`,
                response,
                { input: 'json', output: 'json' }
            );
        }

        // Setting the notification variable to the notification found in the response
        this.variables.set(Notification, notification);
    }
);

Then(
    'der findes en notifikation for date {string} som gemmes som {string}',
    async function (date, Notification) {
        // Getting the notification messages from the last response
        const response =
            this.getLastResponse().body.response.main.Envelope.Body
                .GetMessagesResponse.NotificationMessage;

        // Getting the date from a previous fixture
        date = this.variables.get(date);
        // slice last 10 characters
        date = date.slice(0, 10);

        // Making notification variable with no value
        let notification;

        // If the response is an array
        if (Array.isArray(response)) {
            // Finding the notifications where the data card has been updated and selecting the one with the right date
            notification = await jq.run(
                `.[]|select(.Message.NotifyContent.ConsentUpdatedNotification.date.__value=="${date}")`,
                response,
                { input: 'json', output: 'json' }
            );
            console.log(notification);
        } else {
            // If the response is not an array
            // Finding the notifications where the data card has been updated and selecting the one with the right date
            notification = await jq.run(
                `.|select(.Message.NotifyContent.ConsentUpdatedNotification.date.__value=="${date}")`,
                response,
                { input: 'json', output: 'json' }
            );
        }

        // Setting the notification variable to the notification found in the response
        this.variables.set(Notification, notification);
    }
);

Then(
    'der findes en notifikation for cpr {string} som gemmes som {string}',
    async function (PatientCPR, Notification) {
        // Getting the notification messages from the last response
        const response =
            this.getLastResponse().body.response.main.Envelope.Body
                .GetMessagesResponse.NotificationMessage;

        // Making notification variable with no value
        let notification;

        const cpr = this.replaceVariables(PatientCPR);

        // If the response is an array
        if (Array.isArray(response)) {
            // Finding the notifications where the data card has been updated and selecting the first one with the right cpr
            notification = await jq.run(
                `first(.[] | select(.Message.NotifyContent.__id == "${cpr}"))`,
                response,
                { input: 'json', output: 'json' }
            );
        } else {
            // If the response is not an array
            // Finding the notifications where the data card has been updated and selecting the first one with the right cpr
            notification = await jq.run(
                `.|select(.Message.NotifyContent.__id == "${cpr}")`,
                response,
                { input: 'json', output: 'json' }
            );
        }

        // Setting the notification variable to the notification found in the response
        this.variables.set(Notification, notification);
    }
);

Then('{string} har opdateret behandlingstestamente {string} lig med {string}', async function (Notification, Field, Value) {
    // Getting notification
    let notification = this.variables.get(Notification);

    if (notification[0] != null) {
        notification = await jq.run(
            `.[1]`,
            notification,
            { input: 'json', output: 'json' }
        );
    }
    // Converting value
    const value = this.replaceVariables(Value);

    // If the field is a date (special case)
    if (Field == 'date') {
        // Formating the date-value from the fixture
        let date = parseRelativeDate(value).substring(0, 10);
        console.log("this is date", date);

        // Expecting the field value in the notification to equal the date-value from the fixture
        expect(
            notification.Message.NotifyContent.TreatmentWillUpdated[Field]
                ._text
        ).to.eql(date);
    } else {
        // All other cases
        // Expecting the field value in the notification to equal the value from the fixture
        expect(
            notification.Message.NotifyContent.TreatmentWillUpdated[Field]
                ._text
        ).to.eql(value);
    }
});

Then('{string} har opdateret frabedelse {string} lig med {string}', function (Notification, Field, Value) {
    // Getting notification
    const notification = this.variables.get(Notification);


    // Converting value
    const value = this.replaceVariables(Value);

    // If the field is a date (special case)
    if (Field == 'date') {
        // Formating the date-value from the fixture
        let date = parseRelativeDate(value).substring(0, 10);
        console.log("this is date", date);

        // Expecting the field value in the notification to equal the date-value from the fixture
        expect(
            notification.Message.NotifyContent.ConsentUpdated[Field]
                .__value
        ).to.eql(date);
    } else {
        // All other cases
        // Expecting the field value in the notification to equal the value from the fixture
        expect(
            notification.Message.NotifyContent.ConsentUpdated[Field]
                .__value
        ).to.eql(value);
    }
});

Then('{string} har opdateret organdoner {string} lig med {string}',
    async function (Notification, Field, Value) {
        // Getting notification
        let notification = this.variables.get(Notification);

        // Converting value
        const value = this.replaceVariables(Value);

        if (notification[0] != null) {
            notification = await jq.run(
                `.[1]`,
                notification,
                { input: 'json', output: 'json' }
            );
        }

        // If the field is a date (special case)
        if (Field == 'date') {
            // Formating the date-value from the fixture
            let date = parseRelativeDate(value).substring(0, 10);
            console.log("this is date", date);
            console.log("this is notification", notification);

            expect(
                notification.Message.NotifyContent.OrgandonorUpdated[Field]
                    ._text
            ).to.eql(date);

        } else {
            // All other cases
            // Expecting the field value in the notification to equal the value from the fixture
            expect(
                notification.Message.NotifyContent.OrgandonorUpdated[Field]
                    ._text
            ).to.eql(value);
        }
    }
);


// This fixture expects the notification to have an data-card-updated-field equal to a value
Then(
    '{string} har opdateret stamkort {string} lig med {string}',
    function (Notification, Field, Value) {
        // Getting notification
        const notification = this.variables.get(Notification);

        // Converting value
        const value = this.replaceVariables(Value);

        // If the field is a date (special case)
        if (Field == 'date') {
            // Formating the date-value from the fixture
            let date = parseRelativeDate(value).substring(0, 10);

            // Expecting the field value in the notification to equal the date-value from the fixture
            expect(
                notification.Message.NotifyContent.DataCardUpdated[Field]
                    .__value
            ).to.eql(date);
        } else {
            // All other cases
            // Expecting the field value in the notification to equal the value from the fixture
            expect(
                notification.Message.NotifyContent.DataCardUpdated[Field]
                    .__value
            ).to.eql(value);
        }
    }
);
//#endregion


//#region ODR

Given('organdonerregistrering er oprettet for {string} og messageId er gemt som {string}', async function (PatientCPR, MessageId) {
    PatientCPR = this.replaceVariables(PatientCPR);
    response = await CreateOrganDonorRegistration(this.attach, PatientCPR);

    messageId = response.body.response.main.Envelope.Header.RelatesTo._text;

    this.variables.set(MessageId, messageId);
});

Then('organdoner slettes for {string}', async function (PatientCPR) {
    PatientCPR = this.replaceVariables(PatientCPR);
    response = await DeleteOrganDonorRegistration(this.attach, PatientCPR);
});

//#endregion


//#region BTR

Then('Behandlingstestemente er slettet for {string}', async function (PatientCPR) {
    PatientCPR = this.replaceVariables(PatientCPR);
    await DeleteTreatmentWill(this.attach, PatientCPR);
});

Given('behandlingstestamente er oprettet for {string} og messageId er gemt som {string}', async function (PatientCPR, MessageId) {
    // Converting to CPR
    PatientCPR = this.replaceVariables(PatientCPR);
    console.log(PatientCPR, this.attach);
    // Creating BTR for person 
    const BTR = await CreateTreatmentWill(
        this.attach,
        PatientCPR,
    );
    const messageId =
        BTR.body.response.main.Envelope.Header.RelatesTo._text;

    // Setting message id variable to the message id found in the response
    this.variables.set(MessageId, messageId);
    console.log("this is MessageID", MessageId, "and this messageid", messageId);
});

//#endregion

//#region Related Person
// This fixture creates a related person in SKR and saves the message id
Given(
    'pårørende er oprettet i SKR for {string} og messageId er gemt som {string}',
    // It is an async function because await is used
    async function (PatientCPR, MessageId) {
        // Converting to CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Creating random uniq related person id
        let relatedPersonId = v4();

        // Creating a related person with the new related person id and saving the response
        const relatedPerson = await CreateRelative(
            this.attach,
            PatientCPR,
            relatedPersonId
        );
        console.log("this is related", relatedPerson);

        // Getting the message id from the new related person response. Id found in the header
        const messageId =
            relatedPerson.body.response.main.Envelope.Header.RelatesTo._text;

        // Setting message id variable to the message id found in the response
        this.variables.set(MessageId, messageId);
        console.log("this is MessageID", MessageId, "and this messageid", messageId);
    }
);

// This fixture deletes a related person in SKR and saves the message id
Given(
    'pårørende er slettet i SKR for {string} og messageId er gemt som {string}',
    // It is an async function because await is used
    async function (PatientCPR, MessageId) {
        // Converting to CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting PDC Entries with DDS Repository DGWS
        let pdcEntries = await getPDCEntries(this.attach, PatientCPR, 3);

        // Finding PDC component with related person by using jq
        const pdcComponents = await jq.run(
            // Selects elements in the array with the observation code "RelativeTypedIn". Returning all elements in an array
            `[.[]|select(.observation.code|select(."__code"? =="RelativeTypedIn"))]`,
            pdcEntries,
            { input: 'json', output: 'json' }
        );

        // Deleting a related person from the related-person-array by attaching the related person id from index 1 and patient CPR and saving the response
        const deleleteRelatedPerson = await DeleteRelatives(
            this.attach,
            PatientCPR,
            pdcComponents[0].observation.id.__extension
        );

        // Getting the message id from the deleted related person response. Id found in the header
        const messageId =
            deleleteRelatedPerson.body.response.main.Envelope.Header.RelatesTo
                ._text;

        // Setting message id variable to the messge id found in the response
        this.variables.set(MessageId, messageId);
    }
);

// This fixture updates a related person in SKR and saves the message id
Given(
    'pårørende er opdateret i SKR for {string} og messageId er gemt som {string}',
    // It is an async function because await is used
    async function (PatientCPR, MessageId) {
        // Converting to CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting PDC Entries with DDS Repository DGWS
        let pdcEntries = await getPDCEntries(this.attach, PatientCPR, 3);

        // Finding PDC component with related person by using jq
        const pdcComponents = await jq.run(
            // Selects elements in the array with the observation code "RelativeTypedIn". Returning all elements in an array
            `[.[]|select(.observation.code|select(."__code"? =="RelativeTypedIn"))]`,
            pdcEntries,
            { input: 'json', output: 'json' }
        );

        // Updating a related person from the related-person-array by attaching the related person id from index 1 and patient CPR and saving the response
        const updatedRelative = await UpdateRelatives(
            this.attach,
            PatientCPR,
            pdcComponents[0].observation.id.__extension
        );

        // Getting the message id from the updated related person response. Id found in the header
        const messageId =
            updatedRelative.body.response.main.Envelope.Header.RelatesTo._text;

        // Setting the message id variable to the message id found in the response
        this.variables.set(MessageId, messageId);
    }
);
//#endregion

//#region Temporary Address
// This fixture creates a temporary address in SKR and saves the message id
Given(
    'midlertidig adresse er oprettet i SKR for {string} og messageId er gemt som {string}',
    // Timeout is set to -1 because it takes some time before the message id is available
    { timeout: -1 },
    // It is an async function because await is used
    async function (PatientCPR, MessageId) {
        // Converting to CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Creating random uniq temporary address id
        let tempAdressId = v4();

        // Creating a temporary address with the new temporary address id and saving the response
        const temporaryAddress = await CreateTemporaryAddress(
            this.attach,
            tempAdressId,
            PatientCPR
        );

        // Getting the message id from the new temporary address response. Id found in the header
        const messageId =
            temporaryAddress.body.response.main.Envelope.Header.RelatesTo._text;

        // Setting the message id variable to the message id found in the response
        this.variables.set(MessageId, messageId);
    }
);

// This fixture deletes a temporary address in SKR and saves the message id
Given(
    'midlertidig adresse er slettet i SKR for {string} og messageId er gemt som {string}',
    // Timeout is set to -1 because it takes some time before the message id is available
    { timeout: -1 },
    // It is an async function because await is used
    async function (PatientCPR, MessageId) {
        // Converting to CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting PDC Entries with DDS Repository DGWS
        let pdcEntries = await getPDCEntries(this.attach, PatientCPR, 3);

        // Finding PDC component with temporary address by using jq
        const pdcComponents = await jq.run(
            // Selects element in the array with the observation code "TempAddrTypeIn", returning the element
            `.[]|select(.observation.code.__code=="TempAddrTypedIn")`,
            pdcEntries,
            { input: 'json', output: 'json' }
        );

        // Deleting the temporary address by attaching the temporary address id and patient CPR and saving the response
        const deletedTemporaryAddress = await DeleteTemporaryAddress(
            this.attach,
            pdcComponents.observation.id.__extension,
            PatientCPR
        );

        // Getting the message id from the deleted temporary address response. Id found in the header
        const messageId =
            deletedTemporaryAddress.body.response.main.Envelope.Header.RelatesTo
                ._text;

        // Setting message id variable to the message id found in the response
        this.variables.set(MessageId, messageId);
    }
);

// This fixtures updates a temporary address in SKR and saves the message id
Given(
    'midlertidig adresse er opdateret i SKR for {string} og messageId er gemt som {string}',
    // Timeout is set to -1 because it takes some time before the message id is available
    { timeout: -1 },
    // It is an async function because await is used
    async function (PatientCPR, MessageId) {
        // Converting to CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting PDC Entries with DDS Repository DGWS
        let pdcEntries = await getPDCEntries(this.attach, PatientCPR, 3);

        // Finding PDC component with temporary address by using jq
        const pdcComponents = await jq.run(
            // Selects element in the array with the observation code "TempAddrTypeIn", returning the element
            `.[]|select(.observation.code.__code=="TempAddrTypedIn")`,
            pdcEntries,
            { input: 'json', output: 'json' }
        );

        // Updating the temporary address by attaching the temporary address id and patient CPR and saving the response
        const updatedTemporaryAddress = await UpdateTemporaryAddress(
            this.attach,
            pdcComponents.observation.id.__extension,
            PatientCPR
        );

        // Getting the message id from the deleted temporary address response. Id found in the header
        const messageId =
            updatedTemporaryAddress.body.response.main.Envelope.Header.RelatesTo
                ._text;

        // Setting message id variable to the message id found in the response
        this.variables.set(MessageId, messageId);
    }
);
//#endregion

//#region Language
// This fixture creates a language in SKR and saves the message id
Given(
    'sprog er oprettet i SKR for {string} og messageId er gemt som {string}',
    // Timeout is set to -1 because it takes some time before the message id is available
    { timeout: -1 },
    // It is an async function because await is used
    async function (PatientCPR, MessageId) {
        // Converting to CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Creating random uniq language id
        let languageId = v4();

        // Creating a language with the new language id and saving the response
        const language = await CreateLanguage(
            this.attach,
            languageId,
            PatientCPR
        );

        // Getting the message id from the new language response. Id found in the header
        const messageId =
            language.body.response.main.Envelope.Header.RelatesTo._text;

        // Setting message id variable to the message id found in the response body
        this.variables.set(MessageId, messageId);
    }
);

// This fixture deletes a language in SKR and saves the message id
Given(
    'sprog er slettet i SKR for {string} og messageId er gemt som {string}',
    // Timeout is set to -1 because it takes some time before the message id is available
    { timeout: -1 },
    // It is an async function because await is used
    async function (PatientCPR, MessageId) {
        // Converting to CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting PDC Entries with DDS Repository DGWS
        let pdcEntries = await getPDCEntries(this.attach, PatientCPR, 3);

        // Finding PDC component with language by using jq
        const pdcComponents = await jq.run(
            // Selects element in the array with the observation code "LanguageTypeIn", returning the element
            `.[]|select(.observation.code.__code=="LanguageTypedIn")`,
            pdcEntries,
            { input: 'json', output: 'json' }
        );

        // Deleting the language by attaching the language id and patient CPR
        const deletedLanguage = await DeleteLanguage(
            this.attach,
            pdcComponents.observation.id.__extension,
            PatientCPR
        );

        // Getting the message id from the deleted language response. Id found in the header
        const messageId =
            deletedLanguage.body.response.main.Envelope.Header.RelatesTo._text;

        // Setting message id variable to the message id found in the response
        this.variables.set(MessageId, messageId);
    }
);

// This fixture updates a language in SKR and saves the message id
Given(
    'sprog er opdateret i SKR for {string} og messageId er gemt som {string}',
    // Timeout is set to -1 because it takes some time before the message id is available
    { timeout: -1 },
    // It is an async function because await is used
    async function (PatientCPR, MessageId) {
        // Converting to CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting PDC Entries with DDS Repository DGWS
        let pdcEntries = await getPDCEntries(this.attach, PatientCPR, 3);

        // Finding PDC component with language by using jq
        const pdcComponents = await jq.run(
            // Selects element in the array with the observation code "LanguageTypeIn", returning the element
            `.[]|select(.observation.code.__code=="LanguageTypedIn")`,
            pdcEntries,
            { input: 'json', output: 'json' }
        );

        // Updating the language by attaching the language id and patient CPR
        const updatedLanguage = await UpdateLanguage(
            this.attach,
            pdcComponents.observation.id.__extension,
            PatientCPR
        );

        // Getting the message id from the updated language response. Id found in the header
        const messageId =
            updatedLanguage.body.response.main.Envelope.Header.RelatesTo._text;

        // Setting the message id variable to the message id found in the response
        this.variables.set(MessageId, messageId);
    }
);
//#endregion

//#region Health Provider
// This fixture creates a health provider in SKR and saves the message id
Given(
    'tandlæge er oprettet i SKR for {string} og messageId er gemt som {string}',
    // Timeout is set to -1 because it takes some time before the message id is available
    { timeout: -1 },
    // It is an async function because await is used
    async function (PatientCPR, MessageId) {
        // Converting to CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Creating random uniq health provider id
        let healthProviderId = v4();

        // Creating a health provider with the new health provider id and saving the response
        const healthProvider = await CreateHealthProvider(
            this.attach,
            healthProviderId,
            PatientCPR
        );

        // Getting the message id from the new health provider response. Id found in the header
        const messageId =
            healthProvider.body.response.main.Envelope.Header.RelatesTo._text;

        // Setting message id variable to the message id found in the response
        this.variables.set(MessageId, messageId);
    }
);

// This fixture deletes a health provider in SKR and saves the message id
Given(
    'tandlæge er slettet i SKR for {string} og messageId er gemt som {string}',
    // Timeout is set to -1 because it takes some time before the message id is available
    { timeout: -1 },
    // It is an async function because await is used
    async function (PatientCPR, MessageId) {
        // Converting to CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting PDC Entries with DDS Repository DGWS
        let pdcEntries = await getPDCEntries(this.attach, PatientCPR, 3);

        // Finding PDC component with health provider by using jq
        const pdcComponents = await jq.run(
            // Selects element in the array with the observation code "DentistTypeIn", returning the element
            `.[]|select(.observation.code.__code=="DentistTypedIn")`,
            pdcEntries,
            { input: 'json', output: 'json' }
        );

        // Deleting the health provider by attaching the health provider id and patient CPR and saving the response
        const deletedHealthProvider = await DeleteHealthProvider(
            this.attach,
            pdcComponents.observation.id.__extension,
            PatientCPR
        );

        // Getting the message id from the deleted health provider response. Id found in the header
        const messageId =
            deletedHealthProvider.body.response.main.Envelope.Header.RelatesTo
                ._text;

        // Setting message id variable to the message id found in the response
        this.variables.set(MessageId, messageId);
    }
);

// This fixture opdates a health provider in SKR and saves the message id
Given(
    'tandlæge er opdateret i SKR for {string} og messageId er gemt som {string}',
    // Timeout is set to -1 because it takes some time before the message id is available
    { timeout: -1 },
    // It is an async function because await is used
    async function (PatientCPR, MessageId) {
        // Converting to CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting PDC Entries with DDS Repository DGWS
        let pdcEntries = await getPDCEntries(this.attach, PatientCPR, 3);

        // Finding PDC component with health provider by using jq
        const pdcComponents = await jq.run(
            // Selects element in the array with the observation code "DentistTypeIn", returning the element
            `.[]|select(.observation.code.__code=="DentistTypedIn")`,
            pdcEntries,
            { input: 'json', output: 'json' }
        );

        // Updating the health provider by attaching the health provider id and patient CPR and saving the response
        const deletedHealthProvider = await UpdateHealthProvider(
            this.attach,
            pdcComponents.observation.id.__extension,
            PatientCPR
        );

        // Getting the message id from the updated health provider response. Id found in the header
        const messageId =
            deletedHealthProvider.body.response.main.Envelope.Header.RelatesTo
                ._text;

        // Setting the message id variable to the message id found in the response
        this.variables.set(MessageId, messageId);
    }
);
//#endregion

//#region Contact Information
// This fixture opdates a patients contact information and saves the message id
Given(
    'kontaktinformation er opdateret i SKR for {string} og messageId er gemt som {string}',
    // It is an async function because await is used
    async function (PatientCPR, MessageId) {
        // Converting to CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Updating contact information and saving the response
        const contactInformation = await UpdateContactInformation(
            this.attach,
            PatientCPR
        );

        // Getting the message id from the updated contact information response. Id found in the header
        const messageId =
            contactInformation.body.response.main.Envelope.Header.RelatesTo
                ._text;

        // Setting the message id variable to the message id found in the response
        this.variables.set(MessageId, messageId);
    }
);
//#endregion
