const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const jq = require('node-jq');
const { expect } = require('../util');
const roles = new Map(require('../../config/roles.json'));

Then('retursvar indeholder {string} lig med {string}', async function (element, expectedValue) {
    element = this.replaceVariables(element);
    expectedValue = this.replaceVariables(expectedValue);
    let response = this.getLastResponse().body.response.main.Envelope.Body;

    if (response.PersonLookupResponse != undefined) {
        response = response.PersonLookupResponse.PersonInformationStructure;
    } else if (response.getPersonInformationOut != undefined) {
        response = response.getPersonInformationOut.PersonInformationStructure;
    } else if (response.getPersonInformationWithHistoricOut != undefined) {
        response = response.getPersonInformationWithHistoricOut.PersonInformationStructure;
    } else if (response.getPersonWithHealthCareInformationAndHistoricOut != undefined) {
        response = response.getPersonWithHealthCareInformationAndHistoricOut.PersonWithHealthCareInformationStructure.PersonInformationStructure;
    } else {
        response = response.getPersonWithHealthCareInformationOut
            .PersonWithHealthCareInformationStructure.PersonInformationStructure;
    }

    let infoValue;
    switch (element) {
        case 'PersonGivenName':
        case 'PersonMiddleName':
        case 'PersonSurnameName':
            infoValue = response.RegularCPRPerson.SimpleCPRPerson.PersonNameStructure[element]._text;
            break;
        case 'PersonCivilRegistrationIdentifier':
            infoValue = response.RegularCPRPerson.SimpleCPRPerson.PersonCivilRegistrationIdentifier._text;
            break;
        case 'PersonNameForAddressingName':
        case 'PersonGenderCode':
            infoValue = response.RegularCPRPerson[element]._text;
            break;
        case 'BirthDate':
            infoValue = response.RegularCPRPerson.PersonBirthDateStructure[element]._text;
            break;
        case 'PersonCivilRegistrationStatusCode':
        case 'PersonCivilRegistrationStatusStartDate':
            infoValue = response.RegularCPRPerson.PersonCivilRegistrationStatusStructure[element]._text;
            break;
        case 'StreetName':
        case 'StreetNameForAddressingName':
        case 'PostCodeIdentifier':
        case 'DistrictName':
            infoValue = response.PersonAddressStructure.AddressComplete.AddressPostal[element]._text;
            break;
        case 'MunicipalityCode':
        case 'StreetCode':
        case 'StreetBuildingIdentifier':
            infoValue = response.PersonAddressStructure.AddressComplete.AddressAccess[element]._text;
            break;
        case 'CountyCode':
            infoValue = response.PersonAddressStructure.CountyCode._text;
            break;
        case 'PersonInformationProtectionIndicator':
            infoValue = response.RegularCPRPerson.PersonInformationProtectionIndicator._text;
            break;
        case 'PersonCustodyIdentifier':
        case 'PersonCustodyRelation':
            if (Array.isArray(response.PersonCustodyStructure)) {
                response.PersonCustodyStructure.forEach(pc => {
                    if (pc[element]._text = expectedValue) {
                        infoValue = pc[element]._text
                    }
                })
            } else {
                infoValue = response.PersonCustodyStructure[element]._text
            }
            break;
    }
    expect(infoValue).to.equal(expectedValue);

});

Then('retursvar indeholder praktiserende med {string} lig med {string}', function (element, expectedValue) {
    element = this.replaceVariables(element);
    expectedValue = this.replaceVariables(expectedValue);

    let response = this.getLastResponse().body.response.main.Envelope.Body
    if (response.getPersonWithHealthCareInformationOut != undefined) {
        response = response.getPersonWithHealthCareInformationOut.PersonWithHealthCareInformationStructure.PersonHealthCareInformationStructure;
    } else if (response.getPersonWithHealthCareInformationAndHistoricOut != undefined) {
        response = response.getPersonWithHealthCareInformationAndHistoricOut.PersonWithHealthCareInformationStructure.PersonHealthCareInformationStructure;
    }

    if (element == "PublicHealthInsuranceGroupIdentifier") {
        expect(response.PersonPublicHealthInsurance.PublicHealthInsuranceGroupIdentifier._text).to.equal(expectedValue)
    } else {
        expect(response.CurrentGeneralPractitionerStructure.AssociatedGeneralPractitionerStructure[element]._text).to.equal(expectedValue);
    }
});

Then('retursvar indeholder værge med cpr {string} med relation {string}', function (cpr, relation) {
    cpr = this.replaceVariables(cpr);
    relation = this.replaceVariables(relation);

    let response =
        this.getLastResponse().body.response.main.Envelope.Body

    if (response.PersonLookupResponse != undefined) {
                response = response.PersonLookupResponse.PersonInformationStructure.PersonCustodyStructure;
            } else if(response.getPersonInformationOut != undefined)  {
                response = response.getPersonInformationOut.PersonInformationStructure.PersonCustodyStructure;
            } else {
                response = response.getPersonWithHealthCareInformationOut
                        .PersonWithHealthCareInformationStructure.PersonInformationStructure.PersonCustodyStructure;
                        }

    let custodyId
    let custodyRelation;
    if (response.length > 0) {
        response.forEach(custody => {
            if (custody.PersonCustodyIdentifier._text == cpr) {
                custodyId = custody.PersonCustodyIdentifier._text;
                custodyRelation = custody.PersonCustodyRelation._text
            }
        });
    }
    else {
        custodyId = response.PersonCustodyIdentifier._text;
        custodyRelation = response.PersonCustodyRelation._text;
    }
    expect(custodyId).to.equal(cpr);
    expect(custodyRelation).to.equal(relation);
});

Then('retursvar indeholder myndling med cpr {string} med relation {string}', function (cpr, relation) {
    cpr = this.replaceVariables(cpr);
    relation = this.replaceVariables(relation);

    let response =
            this.getLastResponse().body.response.main.Envelope.Body

        if (response.PersonLookupResponse != undefined) {
                    response = response.PersonLookupResponse.PersonInformationStructure.PersonWardStructure;
                } else if(response.getPersonInformationOut != undefined)  {
                    response = response.getPersonInformationOut.PersonInformationStructure.PersonWardStructure;
                } else {
                    response = response.getPersonWithHealthCareInformationOut
                            .PersonWithHealthCareInformationStructure.PersonInformationStructure.PersonWardStructure;
                            }
    let wardId
    let wardRelation;
    if (response.length > 0) {
        response.forEach(ward => {
            if (ward.PersonWardIdentifier._text == cpr) {
                wardId = ward.PersonWardIdentifier._text;
                wardRelation = ward.PersonWardRelation._text
            }
        });
    }
    else {
        wardId = response.PersonWardIdentifier._text;
        wardRelation = response.PersonWardRelation._text;
    }
    expect(wardId).to.equal(cpr);
    expect(wardRelation).to.equal(relation);
});

Then('retursvar indeholder antal personer lig med {int}', function (int) {
    const response =
        this.getLastResponse().body.response.main.Envelope.Body.PersonLookupResponse.PersonInformationStructure;
    expect(response.length).to.equal(4);
});
