const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const jq = require('node-jq');
const { expect } = require('../util');
const roles = new Map(require('../../config/roles.json'));
const fs = require('fs/promises');
const path = require('path');

Then('feed indeholder {string} lig med {string}', function (element, responsesvar) {
    element = this.replaceVariables(element);
    responsesvar = this.replaceVariables(responsesvar);

    const feed = this.getLastResponse().body.response.main.Envelope.Body.ReplicationResponse.feed;

    const basePersonPath = ['entry', 'content', 'person'];
    const fields = [
        'koen', 'fornavn', 'mellemnavn', 'efternavn', 'vejnavn',
        'status', 'validTo', 'validFrom', 'cpr', 'standardAdresse',
        'husnummer', 'postnummer', 'postdistrikt', 'statusDato',
        'foedselsdato', 'gaeldendeCPR', 'vejKode', 'kommuneKode',
        'startDato', 'navnStartDato', 'adresseringsNavn', 'stilling'
    ];

    const pathMap = Object.fromEntries(
        fields.map(key => [key, [...basePersonPath, key, '_text']])
    );

    pathMap.author = ['author', 'name', '_text'];
    pathMap.title = ['title', '_text'];

    const getValueFromPath = (obj, path) =>
        path.reduce((acc, key) => acc?.[key], obj);

    if (element === 'entry') {
        const entries = feed.entry;
        const length = Array.isArray(entries) ? entries.length : entries ? 1 : 0;
        expect(length.toString()).to.equal(responsesvar);
    } else if (pathMap[element]) {
        const actualValue = getValueFromPath(feed, pathMap[element]);
        expect(actualValue).to.equal(responsesvar);
    } else {
        throw new Error(`No match for '${element}'`);
    }
});

function formatXml(xml) {
    const reg = /(>)(<)(\/*)/g;
    xml = xml.replace(/\r?\n/g, '').replace(reg, '$1\n$2$3');
    return xml.trim();
}

async function loadExpectedXml(cpr) {
    const filePath = path.resolve(__dirname, `../saveDataCard-check/${cpr}`);
    return await fs.readFile(filePath, 'utf8');
}

function xmlMapping(xml) {
    return xml
        .split(/\r?\n/)
        .map(line => line.replace(/\s+/g, ''))
        .map(line => line.trim())
}

Then('personens stamkort {string} sammenlignes med gemt stamkort for {string}', async function (response, cpr) {
    cpr = this.replaceVariables(cpr);
    response = this.replaceVariables(response).body.output;

    const formattedXml = formatXml(response);
    const expectedRes = await loadExpectedXml(cpr)
    const linesFromRes = xmlMapping(formattedXml)
    const linesFromExpectedRes = xmlMapping(expectedRes)

    const attachment = [];
    linesFromRes.forEach((line, i) => {
        if (line !== linesFromExpectedRes[i]) {
            attachment.push(`Difference at line: ${i + 1}`);
            attachment.push("New: " + line);
            attachment.push("Saved: " + linesFromExpectedRes[i]);
            attachment.push('____________________________________');
        }
    });

    if (attachment.length > 0) {
        this.attach(attachment.join('\n'), 'text/plain');
    }
});

Then('personens stamkort {string} er lig med gemt stamkort for {string}', async function (response, cpr) {
    cpr = this.replaceVariables(cpr);
    response = this.replaceVariables(response).body.output;

    //Clinical document
    const formattedXml = formatXml(response).match(/<ClinicalDocument[\s\S]*?<\/ClinicalDocument>/);
    const expectedRes = await loadExpectedXml(cpr)
    const pdc = expectedRes.match(/<ClinicalDocument[\s\S]*?<\/ClinicalDocument>/);
    const linesFromRes = xmlMapping(pdc ? pdc[0] : '')
    const linesFromExpectedRes = xmlMapping(formattedXml ? formattedXml[0] : '')

    linesFromRes.forEach((line, i) => {
        if (!line.includes('effectiveTimevalue') && !linesFromExpectedRes[i].includes('effectiveTimevalue')) {
            expect(line).to.eql(linesFromExpectedRes[i]);
        }
    });
});