const { setWorldConstructor, World } = require('@cucumber/cucumber');
const v4 = require('uuid').v4;
const { parseRelativeDate, oid } = require('./util');

class CustomWorld extends World {
    datasets = new Map();
    documents = [];
    responses = [];
    variables = new Map();

    constructor(options) {
        // needed so `attach`, `log` and `parameters` are properly set
        super(options);
    }

    getLastDocument() {
        if (this.documents.length == 0) {
            throw new Error('No documents have been saved');
        }
        return this.documents[this.documents.length - 1];
    }

    getLastResponse() {
        if (this.responses.length == 0) {
            throw new Error('No responses have been saved');
        }
        return this.responses[this.responses.length - 1];
    }

    replaceVariables(value) {
        if (value.startsWith('$')) {
            const val = this.variables.get(value.slice(1));
            if (val) {
                return val;
            }
        }
        return value;
    }

    parseDataTable(dataTable) {
        const data = dataTable.rowsHash();
        for (const [key] of Object.entries(data)) {
            data[key] = parseRelativeDate(data[key]);
            data[key] = this.replaceVariables(data[key]);

            if (data[key].match(/nytuniktid/i)) {
                data[key] = v4();
            }
            if (data[key].match(/nytuniktoid/i)) {
                data[key] = oid();
            }
            if (data[key].match(/^null$/i) || data[key] === null) {
                data[key] = '';
            }
        }
        return data;
    }
}

setWorldConstructor(CustomWorld);
